@@ -38,7 +38,10 @@ static SyncSource *createSource(const SyncSourceParams &params)
 
     isMe = sourceType.m_backend == "Evolution Task List";
     if (isMe || sourceType.m_backend == "todo") {
-        if (sourceType.m_format == "" || sourceType.m_format == "text/calendar") { 
+        if (sourceType.m_format == "" ||
+            sourceType.m_format == "text/calendar" ||
+            sourceType.m_format == "text/x-calendar" ||
+            sourceType.m_format == "text/x-vcalendar") {
             return
 #ifdef ENABLE_ECAL
                 enabled ? new EvolutionCalendarSource(E_CAL_SOURCE_TYPE_TODO, params) :
@@ -68,8 +71,10 @@ static SyncSource *createSource(const SyncSourceParams &params)
 
     isMe = sourceType.m_backend == "Evolution Calendar";
     if (isMe || sourceType.m_backend == "calendar") {
-        if (sourceType.m_format == "" || sourceType.m_format == "text/calendar" ||
-            sourceType.m_format == "text/x-vcalendar" /* this is for backwards compatibility with broken configs */ ) {
+        if (sourceType.m_format == "" ||
+            sourceType.m_format == "text/calendar" ||
+            sourceType.m_format == "text/x-calendar" ||
+            sourceType.m_format == "text/x-vcalendar") {
             return
 #ifdef ENABLE_ECAL
                 enabled ? new EvolutionCalendarSource(E_CAL_SOURCE_TYPE_EVENT, params) :
@@ -92,14 +97,14 @@ static RegisterSyncSource registerMe("Evolution Calendar/Task List/Memos",
                                      createSource,
                                      "Evolution Calendar = calendar = events = evolution-events\n"
                                      "   iCalendar 2.0 (default) = text/calendar\n"
-                                     "   vCalendar 1.0 = text/x-calendar\n"
+                                     "   vCalendar 1.0 = text/x-vcalendar\n"
                                      "Evolution Task List = Evolution Tasks = todo = tasks = evolution-tasks\n"
                                      "   iCalendar 2.0 (default) = text/calendar\n"
-                                     "   vCalendar 1.0 = text/x-calendar\n"
+                                     "   vCalendar 1.0 = text/x-vcalendar\n"
                                      "Evolution Memos = memo = memos = evolution-memos\n"
                                      "   plain text in UTF-8 (default) = text/plain\n"
                                      "   iCalendar 2.0 = text/calendar\n"
-                                     "   vCalendar 1.0 = text/x-calendar\n"
+                                     "   vCalendar 1.0 = text/x-vcalendar\n"
                                      "   The later format is not tested because none of the\n"
                                      "   supported SyncML servers accepts it.\n",
                                      Values() +
@@ -168,7 +173,7 @@ protected:
         }
 
         boost::shared_ptr<TestingSyncSource> source;
-        source.reset((TestingSyncSource *)SyncSource::createTestingSource("ical20", "evolution-calendar", true, prefix));
+        source.reset((TestingSyncSource *)SyncSource::createTestingSource("eds_event", "evolution-calendar", true, prefix));
         CPPUNIT_ASSERT_NO_THROW(source->open());
 
         string newyork = 
@@ -308,7 +313,7 @@ namespace {
 
 static class iCal20Test : public RegisterSyncSourceTest {
 public:
-    iCal20Test() : RegisterSyncSourceTest("ical20", "ical20") {}
+    iCal20Test() : RegisterSyncSourceTest("eds_event", "eds_event") {}
 
     virtual void updateConfig(ClientTestConfig &config) const
     {
@@ -318,7 +323,7 @@ public:
 
 static class iTodo20Test : public RegisterSyncSourceTest {
 public:
-    iTodo20Test() : RegisterSyncSourceTest("itodo20", "itodo20") {}
+    iTodo20Test() : RegisterSyncSourceTest("eds_task", "eds_task") {}
 
     virtual void updateConfig(ClientTestConfig &config) const
     {
@@ -326,9 +331,21 @@ public:
     }
 } iTodo20Test;
 
+static class SuperTest : public RegisterSyncSourceTest {
+public:
+    SuperTest() : RegisterSyncSourceTest("calendar+todo", "calendar+todo") {}
+
+    virtual void updateConfig(ClientTestConfig &config) const
+    {
+        config.type = "virtual:text/x-vcalendar";
+        config.subConfigs = "eds_event,eds_task";
+    }
+
+} superTest;
+
 static class MemoTest : public RegisterSyncSourceTest {
 public:
-    MemoTest() : RegisterSyncSourceTest("text", "text") {}
+    MemoTest() : RegisterSyncSourceTest("eds_memo", "eds_memo") {}
 
     virtual void updateConfig(ClientTestConfig &config) const
     {
