@@ -191,22 +191,22 @@ void EvolutionContactSource::open()
     // users are not expected to configure an authentication method,
     // so pick one automatically if the user indicated that he wants authentication
     // by setting user or password
-    const char *user = getUser(),
-        *passwd = getPassword();
-    if ((user && user[0]) || (passwd && passwd[0])) {
+    std::string user = getUser(),
+        passwd = getPassword();
+    if (!user.empty() || !passwd.empty()) {
         GList *authmethod;
         if (!e_book_get_supported_auth_methods(m_addressbook, &authmethod, &gerror)) {
             throwError("getting authentication methods", gerror );
         }
         while (authmethod) {
             const char *method = (const char *)authmethod->data;
-            SE_LOG_DEBUG(this, NULL, "%s: trying authentication method \"%s\", user %s, password %s",
-                      getName(), method,
-                      user && user[0] ? "configured" : "not configured",
-                      passwd && passwd[0] ? "configured" : "not configured");
+            SE_LOG_DEBUG(this, NULL, "trying authentication method \"%s\", user %s, password %s",
+                         method,
+                         !user.empty() ? "configured" : "not configured",
+                         !passwd.empty() ? "configured" : "not configured");
             if (e_book_authenticate_user(m_addressbook,
-                                         user ? user : "",
-                                         passwd ? passwd : "",
+                                         user.c_str(),
+                                         passwd.c_str(),
                                          method,
                                          &gerror)) {
                 SE_LOG_DEBUG(this, NULL, "authentication succeeded");
@@ -302,10 +302,6 @@ void EvolutionContactSource::readItem(const string &luid, std::string &item, boo
         throwError(string("reading contact ") + luid,
                    gerror);
     }
-#ifdef ENABLE_MAEMO
-    // load detached photos etc
-    e_contact_inline_data(contact);
-#endif
     eptr<EContact, GObject> contactptr(contact, "contact");
     eptr<char> vcardstr(e_vcard_to_string(&contactptr->parent,
                                           EVC_FORMAT_VCARD_30));
@@ -354,8 +350,8 @@ void EvolutionContactSource::removeItem(const string &uid)
     if (!e_book_remove_contact(m_addressbook, uid.c_str(), &gerror)) {
         if (gerror->domain == E_BOOK_ERROR &&
             gerror->code == E_BOOK_ERROR_CONTACT_NOT_FOUND) {
-            SE_LOG_DEBUG(this, NULL, "%s: %s: request to delete non-existant contact ignored",
-                         getName(), uid.c_str());
+            SE_LOG_DEBUG(this, NULL, "%s: request to delete non-existant contact ignored",
+                         uid.c_str());
             g_clear_error(&gerror);
         } else {
             throwError( string( "deleting contact " ) + uid,
@@ -414,7 +410,7 @@ std::string EvolutionContactSource::getDescription(const string &luid)
     }
 }
 
-const char *EvolutionContactSource::getMimeType() const
+std::string EvolutionContactSource::getMimeType() const
 {
     switch( m_vcardFormat ) {
      case EVC_FORMAT_VCARD_21:
@@ -427,7 +423,7 @@ const char *EvolutionContactSource::getMimeType() const
     }
 }
 
-const char *EvolutionContactSource::getMimeVersion() const
+std::string EvolutionContactSource::getMimeVersion() const
 {
     switch( m_vcardFormat ) {
      case EVC_FORMAT_VCARD_21:
