@@ -53,8 +53,8 @@ class EvolutionContactSource : public EvolutionSyncSource,
     virtual void open();
     virtual bool isEmpty();
     virtual void close();
-    virtual const char *getMimeType() const;
-    virtual const char *getMimeVersion() const;
+    virtual std::string getMimeType() const;
+    virtual std::string getMimeVersion() const;
    
   protected:
     //
