@@ -63,8 +63,7 @@ static RegisterSyncSource registerMe("Evolution Address Book",
                                      "   vCard 2.1 (default) = text/x-vcard\n"
                                      "   vCard 3.0 = text/vcard\n"
                                      "   The later is the internal format of Evolution and preferred with\n"
-                                     "   servers that support it. One such server is ScheduleWorld\n"
-                                     "   together with the \"card3\" uri.\n",
+                                     "   servers that support it.",
                                      Values() +
                                      (Aliases("Evolution Address Book") + "Evolution Contacts" + "evolution-contacts"));
 
@@ -147,35 +146,14 @@ namespace {
 }
 #endif
 
-/**
- * We are using the vcard30 tests because they are 
- * a bit more complete than the vcard21 ones.
- * This also tests the vCard 3.0 <-> vCard 2.1
- * conversion.
- *
- * The local tests become identical with the
- * vCard 3.0 ones because they import/export the
- * same data. 
- */
-static class VCard21Test : public RegisterSyncSourceTest {
-public:
-    VCard21Test() : RegisterSyncSourceTest("vcard21", "vcard30") {}
-
-    virtual void updateConfig(ClientTestConfig &config) const
-    {
-        config.uri = "card"; // Funambol
-        config.type = "evolution-contacts:text/x-vcard";
-        config.testcases = "testcases/vcard30.vcf";
-    }
-} vCard21Test;
-
 static class VCard30Test : public RegisterSyncSourceTest {
 public:
-    VCard30Test() : RegisterSyncSourceTest("vcard30", "vcard30") {}
+    VCard30Test() : RegisterSyncSourceTest("eds_contact", "eds_contact") {}
 
     virtual void updateConfig(ClientTestConfig &config) const
     {
         config.type = "evolution-contacts:text/vcard";
+        config.update = config.genericUpdate;
     }
 } vCard30Test;
 
