@@ -46,6 +46,14 @@ class EvolutionSyncSource : public TrackingSyncSource
         {
         }
 
+    void getSynthesisInfo(SynthesisInfo &info,
+                          XMLConfigFragments &fragments)
+    {
+        TrackingSyncSource::getSynthesisInfo(info, fragments);
+        info.m_backendRule = "EVOLUTION";
+        info.m_datastoreOptions += "      <updateallfields>true</updateallfields>\n";
+    }
+
   protected:
 #ifdef HAVE_EDS
     /**
