@@ -22,6 +22,7 @@
 
 #include <syncevo/SyncConfig.h>
 #include <syncevo/FilterConfigNode.h>
+#include <syncevo/util.h>
 
 #include <set>
 using namespace std;
@@ -33,9 +34,36 @@ using namespace std;
 SE_BEGIN_CXX
 
 class SyncSource;
+class SyncSourceRaw;
 class SyncContext;
 class CmdlineTest;
 
+/**
+ * encodes a locally unique ID (LUID) in such a
+ * way that it is treated as a plain word by shells
+ */
+class CmdlineLUID
+{
+    string m_encodedLUID;
+
+public:
+    /** fill with encoded LUID */
+    void setEncoded(const string &encodedLUID) { m_encodedLUID = encodedLUID; }
+
+    /** return encoded LUID as string */
+    string getEncoded() const { return m_encodedLUID; }
+
+    /** return original LUID */
+    string toLUID() const { return toLUID(m_encodedLUID); }
+    static string toLUID(const string &encoded) { return StringEscape::unescape(encoded, '%'); }
+
+    /** fill with unencoded LUID */
+    void setLUID(const string &luid) { m_encodedLUID = fromLUID(luid); }
+
+    /** convert from unencoded LUID */
+    static string fromLUID(const string &luid) { return StringEscape::escape(luid, '%', StringEscape::STRICT); }
+};
+
 class Cmdline {
 public:
     /**
@@ -44,6 +72,7 @@ public:
      */
     Cmdline(int argc, const char * const *argv, ostream &out, ostream &err);
     Cmdline(const vector<string> &args, ostream &out, ostream &err);
+    Cmdline(ostream &out, ostream &err, const char *arg, ...);
 
     /**
      * parse the command line options
@@ -69,6 +98,15 @@ public:
 
     bool run();
 
+    /**
+     * sync report as owned by this instance, not filled in unless
+     * run() executed a sync
+     */
+    const SyncReport &getReport() const { return m_report; }
+
+    /** the run() call modified configurations (added, updated, removed) */
+    bool configWasModified() const { return m_configModified; }
+
     /**
      * Acts like a boolean, but in addition, can also tell whether the
      * value was explicitly set.
@@ -109,6 +147,9 @@ protected:
     //array to store pointers of arguments
     boost::scoped_array<const char *> m_argvArray;
 
+    /** result of sync, if one was executed */
+    SyncReport m_report;
+
     Bool m_quiet;
     Bool m_dryrun;
     Bool m_status;
@@ -126,31 +167,66 @@ protected:
     Bool m_keyring;
     Bool m_monitor;
     Bool m_useDaemon;
-    FilterConfigNode::ConfigFilter m_syncProps, m_sourceProps;
+    FullProps m_props;
     const ConfigPropertyRegistry &m_validSyncProps;
     const ConfigPropertyRegistry &m_validSourceProps;
 
     string m_restore;
     Bool m_before, m_after;
 
+    Bool m_accessItems;
+    string m_itemPath;
+    string m_delimiter;
+    list<string> m_luids;
+    Bool m_printItems, m_update, m_import, m_export, m_deleteItems;
+
     string m_server;
     string m_template;
     set<string> m_sources;
 
+    /** running the command line modified configuration settings (add, update, remove) */
+    Bool m_configModified;
+
     /** compose description of cmd line option with optional parameter */
     static string cmdOpt(const char *opt, const char *param = NULL);
 
+    /**
+     * rename file or directory by appending .old or (if that already
+     * exists) .old.x for x >= 1; updates config to point to the renamed directory
+     */
+    void makeObsolete(boost::shared_ptr<SyncConfig> &from);
+
+    /**
+     * Copy from one config into another, with filters
+     * applied for the target. All sources are copied
+     * if selectedSources is empty, otherwise only
+     * those.
+     */
+    void copyConfig(const boost::shared_ptr<SyncConfig> &from,
+                    const boost::shared_ptr<SyncConfig> &to,
+                    const set<string> &selectedSources);
+
+    /**
+     * flush, move .synthesis dir, set ConsumerReady, ...
+     */
+    void finishCopy(const boost::shared_ptr<SyncConfig> &from,
+                    const boost::shared_ptr<SyncContext> &to);
+
+    /**
+     * migrate peer config; target context must be ready
+     */
+    void migratePeer(const std::string &fromPeer, const std::string &toPeer);
+
     /**
      * parse sync or source property
      *
-     * @param validProps     list of valid properties
-     * @retval props         add property name/value pair here
+     * @param propertyType   sync, source, or unknown (in which case the property name must be given and must be unique)
      * @param opt            command line option as it appeard in argv (e.g. --sync|--sync-property|-z)
      * @param param          the parameter following the opt, may be NULL if none given (error!)
      * @param propname       if given, then this is the property name and param contains the param value (--sync <param>)
      */
-    bool parseProp(const ConfigPropertyRegistry &validProps,
-                   FilterConfigNode::ConfigFilter &props,
+    
+    bool parseProp(PropertyType propertyType,
                    const char *opt,
                    const char *param,
                    const char *propname = NULL);
@@ -162,22 +238,8 @@ protected:
     bool listProperties(const ConfigPropertyRegistry &validProps,
                         const string &opt);
 
-    typedef map<string, ConfigProps> SourceFilters_t;
-
     /**
-     * read properties from context, then update with
-     * command line properties
-     *
-     * @param context         context name, without @ sign
-     * @retval syncFilter     global sync properties
-     * @retval sourceFilters  entries for specific sources, key "" as fallback
-     */
-    void getFilters(const string &context,
-                    ConfigProps &syncFilter,
-                    SourceFilters_t &sourceFilters);
-
-    /**
-     * check that m_syncProps and m_sourceProps don't contain
+     * check that m_props don't contain
      * properties which only apply to peers, throw error
      * if found
      */
@@ -249,6 +311,22 @@ protected:
     bool parseBool(int opt, const char *longName, const char *shortName,
                    bool def, Bool &value,
                    bool &ok);
+
+    /**
+     * Fill list with all local IDs of the given source.
+     * Unsafe characters are escaped with SafeConfigNode::escape(true,true).
+     * startDataRead() must have been called.
+     */
+    void readLUIDs(SyncSource *source, list<string> &luids);
+
+    /**
+     * Add or update one item.
+     * @param source     SyncSource in write mode (startWriteData must have been called)
+     * @param luid       local ID, empty if item is to be added
+     * @param data       the item data to insert
+     * @return encoded luid of inserted item
+     */
+    CmdlineLUID insertItem(SyncSourceRaw *source, const string &luid, const string &data);
 };
 
 
