@@ -34,14 +34,11 @@ CurlTransportAgent::CurlTransportAgent() :
     m_easyHandle(easyInit()),
     m_slist(NULL),
     m_status(INACTIVE),
-    m_cb(NULL),
+    m_timeoutSeconds(0),
     m_reply(NULL),
     m_replyLen(0),
     m_replySize(0)
 {
-#ifdef ENABLE_MAEMO /* hack because Maemo doesn't support IPv6 yet */
-    curl_easy_setopt(m_easyHandle, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
-#endif
     /*
      * set up for post where message is pushed into curl via
      * its read callback and reply is stored in write callback
@@ -146,11 +143,9 @@ void CurlTransportAgent::setSSL(const std::string &cacerts,
     checkCurl(code);
 }
 
-void CurlTransportAgent::setCallback (TransportCallback cb, void *udata, int interval)
+void CurlTransportAgent::setTimeout(int seconds)
 {
-    m_cb = cb;
-    m_cbData = udata;
-    m_cbInterval = interval;
+    m_timeoutSeconds = seconds;
 }
 
 void CurlTransportAgent::shutdown()
@@ -182,8 +177,8 @@ void CurlTransportAgent::send(const char *data, size_t len)
     m_slist = curl_slist_append(m_slist, contentHeader.c_str());
 
     m_status = ACTIVE;
-    if(m_cb){
-        m_sendStartTime = time(NULL);
+    if (m_timeoutSeconds) {
+        m_sendStartTime = Timespec::monotonic();
     }
     m_aborting = false;
     if ((code = curl_easy_setopt(m_easyHandle, CURLOPT_PROGRESSDATA, static_cast<void *> (this)))||
@@ -294,17 +289,10 @@ int CurlTransportAgent::progressCallback(void* transport, double, double, double
 
 int CurlTransportAgent::processCallback()
 {
-    if (m_cb){   
-        time_t curTime = time(NULL);
-        if (curTime - m_sendStartTime > m_cbInterval){
-            //change here to avoid duplicate call back to the upper layer
-            m_sendStartTime = curTime; 
-            bool cont = m_cb (m_cbData);
-            if (cont) {
-                m_status = TIME_OUT;
-            }else {
-                m_aborting = true;
-            }
+    if (m_timeoutSeconds) {
+        Timespec curTime = Timespec::monotonic();
+        if (curTime > m_sendStartTime + m_timeoutSeconds) {
+            m_status = TIME_OUT;
             return -1;
         }
     }
