@@ -36,14 +36,12 @@ SE_BEGIN_CXX
 SoupTransportAgent::SoupTransportAgent(GMainLoop *loop) :
     m_session(soup_session_async_new()),
     m_loop(loop ?
-           loop :
+           g_main_loop_ref(loop) :
            g_main_loop_new(NULL, TRUE),
            "Soup main loop"),
     m_status(INACTIVE),
-    m_abortEventSource(-1),
-    m_cbEventSource(-1),
-    m_cb(NULL),
-    m_response(NULL)
+    m_timeoutSeconds(0),
+    m_response(0)
 {
 #ifdef HAVE_LIBSOUP_SOUP_GNOME_FEATURES_H
     // use default GNOME proxy settings
@@ -68,10 +66,16 @@ void SoupTransportAgent::setURL(const std::string &url)
 
 void SoupTransportAgent::setProxy(const std::string &proxy)
 {
-    eptr<SoupURI, SoupURI, GLibUnref> uri(soup_uri_new(proxy.c_str()), "Proxy URI");
-    g_object_set(m_session.get(),
-                 SOUP_SESSION_PROXY_URI, uri.get(),
-                 NULL);
+    if (!proxy.empty()) {
+        eptr<SoupURI, SoupURI, GLibUnref> uri(soup_uri_new(proxy.c_str()), "Proxy URI");
+        g_object_set(m_session.get(),
+                     SOUP_SESSION_PROXY_URI, uri.get(),
+                     NULL);
+    } else {
+        g_object_set(m_session.get(),
+                     SOUP_SESSION_PROXY_URI, NULL,
+                     NULL);
+    }
 }
 
 void SoupTransportAgent::setProxyAuth(const std::string &user, const std::string &password)
@@ -110,11 +114,9 @@ void SoupTransportAgent::setUserAgent(const std::string &agent)
                  NULL);
 }
 
-void SoupTransportAgent::setCallback (TransportCallback cb, void *data, int interval)
+void SoupTransportAgent::setTimeout(int seconds)
 {
-    m_cb = cb;
-    m_cbData = data;
-    m_cbInterval = interval;
+    m_timeoutSeconds = seconds;
 }
 
 void SoupTransportAgent::send(const char *data, size_t len)
@@ -141,10 +143,10 @@ void SoupTransportAgent::send(const char *data, size_t len)
     soup_message_set_request(message.get(), m_contentType.c_str(),
                              SOUP_MEMORY_TEMPORARY, data, len);
     m_status = ACTIVE;
-    m_abortEventSource = g_timeout_add_seconds(ABORT_CHECK_INTERVAL, (GSourceFunc) AbortCallback, static_cast<gpointer> (this));
-    if(m_cb){
+    m_abortEventSource = g_timeout_add_seconds(ABORT_CHECK_INTERVAL, AbortCallback, static_cast<gpointer> (this));
+    if (m_timeoutSeconds) {
         m_message = message.get();
-        m_cbEventSource = g_timeout_add_seconds(m_cbInterval, (GSourceFunc) TimeoutCallback, static_cast<gpointer> (this));
+        m_timeoutEventSource = g_timeout_add_seconds(m_timeoutSeconds, TimeoutCallback, static_cast<gpointer> (this));
     }
     soup_session_queue_message(m_session.get(), message.release(),
                                SessionCallback, static_cast<gpointer>(this));
@@ -158,7 +160,7 @@ void SoupTransportAgent::cancel()
       g_main_loop_quit(m_loop.get());
 }
 
-TransportAgent::Status SoupTransportAgent::wait()
+TransportAgent::Status SoupTransportAgent::wait(bool noReply)
 {
     if (!m_failure.empty()) {
         std::string failure;
@@ -167,6 +169,9 @@ TransportAgent::Status SoupTransportAgent::wait()
     }
 
     switch (m_status) {
+    case CLOSED:
+        return CLOSED;
+        break;
     case ACTIVE:
         // block in main loop until our HandleSessionCallback() stops the loop
         g_main_loop_run(m_loop.get());
@@ -189,8 +194,8 @@ TransportAgent::Status SoupTransportAgent::wait()
         SE_THROW_EXCEPTION(TransportException, failure);
     }
 
-    g_source_remove(m_abortEventSource);
-    g_source_remove(m_cbEventSource);
+    m_abortEventSource.set(0);
+    m_timeoutEventSource.set(0);
     return m_status;
 }
 
@@ -250,7 +255,7 @@ void SoupTransportAgent::HandleSessionCallback(SoupSession *session,
 
 gboolean SoupTransportAgent::AbortCallback(gpointer transport)
 {
-    SuspendFlags& s_flags = SyncContext::getSuspendFlags();
+    const SuspendFlags &s_flags = SyncContext::getSuspendFlags();
 
     if (s_flags.state == SuspendFlags::CLIENT_ABORT)
     {
@@ -262,15 +267,10 @@ gboolean SoupTransportAgent::AbortCallback(gpointer transport)
 
 gboolean SoupTransportAgent::processCallback()
 {
-    bool cont = m_cb(m_cbData);
-    if(cont){
-        //stop the message processing and mark status as timeout
-        guint message_status = SOUP_STATUS_CANCELLED;
-        soup_session_cancel_message(m_session.get(), m_message, message_status);
-        m_status = TIME_OUT;
-    }else {
-        cancel();
-    }
+    //stop the message processing and mark status as timeout
+    guint message_status = SOUP_STATUS_CANCELLED;
+    soup_session_cancel_message(m_session.get(), m_message, message_status);
+    m_status = TIME_OUT;
     return FALSE;
 }
 
