@@ -89,12 +89,18 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     bool m_quiet;
     bool m_dryrun;
 
+    bool m_localSync;
+    string m_localPeerContext; /**< context name (including @) if doing local sync */
+    string m_localClientRootPath;
     bool m_serverMode;
+    bool m_serverAlerted;      /**< sync was initiated by server (applies to client and server mode) */
+    bool m_configNeeded;
     std::string m_sessionID;
     SharedBuffer m_initialMessage;
     string m_initialMessageType;
     string m_syncDeviceID;
 
+    FullProps m_configFilters;
     
     boost::shared_ptr<TransportAgent> m_agent;
     /**
@@ -164,6 +170,26 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      */
     bool m_remoteInitiated;
   public:
+    /**
+     * Common initialization code which needs to be done once
+     * at the start of main() in any application using SyncEvolution.
+     * For example, initializes (if applicable) glib and EDS.
+     *
+     * @param appname     static string, must remain valid, defines name of executable (see g_set_prgname())
+     */
+    static void initMain(const char *appname);
+
+    /**
+     * true if binary was compiled as stable release
+     * (see gen-autotools.sh)
+     */
+    static bool isStableRelease();
+
+    /**
+     * override stable release mode (for testing purposes)
+     */
+    static void setStableRelease(bool isStableRelease);
+
     /**
      * SyncContext using a volatile config
      * and no logging.
@@ -171,11 +197,32 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     SyncContext();
 
     /**
-     * @param server     identifies the server config to be used
-     * @param doLogging  write additional log and datatbase files about the sync
+     * Constructor for syncing with a SyncML peer.
+     *
+     * @param peer       identifies the client or server config to be used
+     * @param doLogging  write additional log and datatbase files about the sync;
+     *                   true for regular syncs, false for debugging
      */
     SyncContext(const string &server,
                 bool doLogging = false);
+
+    /**
+     * Constructor for client in a local sync.
+     *
+     * @param client     identifies the client context to be used (@foobar)
+     * @param server     identifies the server peer (foo@bar)
+     * @param rootPath   use this directory as config directory for the
+     *                   peer-specific files (located inside peer directory
+     *                   of server config)
+     * @param agent      transport agent, ready for communication with server
+     * @param doLogging  write additional log and datatbase files about the sync
+     */
+    SyncContext(const string &client,
+                const string &server,
+                const string &rootPath,
+                const boost::shared_ptr<TransportAgent> &agent,
+                bool doLogging = false);
+
     ~SyncContext();
 
     /**
@@ -191,6 +238,30 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     bool getDryRun() { return m_dryrun; }
     void setDryRun(bool dryrun) { m_dryrun = dryrun; }
 
+    bool isLocalSync() const { return m_localSync; }
+
+    bool isServerAlerted() const { return m_serverAlerted; }
+    void setServerAlerted(bool serverAlerted) { m_serverAlerted = serverAlerted; }
+
+    /**
+     * Running operations typically checks that a config really exists
+     * on disk. Setting false disables the check.
+     */
+    bool isConfigNeeded() const { return m_configNeeded; }
+    void setConfigNeeded(bool configNeeded) { m_configNeeded = configNeeded; }
+
+    /**
+     * throws error if config is needed and not available
+     */
+    void checkConfig() const;
+
+    /**
+     * Sets configuration filters. Currently only used in local sync
+     * to configure the sync client.
+     */
+    void setConfigProps(const FullProps &props) { m_configFilters = props; }
+    const FullProps &getConfigProps() const { return m_configFilters; }
+
     /** only for server: device ID of peer */
     void setSyncDeviceID(const std::string &deviceID) { m_syncDeviceID = deviceID; }
     std::string getSyncDeviceID() const { return m_syncDeviceID; }
@@ -312,7 +383,7 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     void checkStatus(SyncReport &report);
 
     /**
-     * throws a runtime_error with the given string
+     * throws a StatusException with a local, fatal error with the given string
      * or (on the iPhone, where exception handling is not
      * supported by the toolchain) prints an error directly
      * and aborts
@@ -323,6 +394,19 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      */
     static void throwError(const string &error);
 
+    /**
+     * throw an exception with a specific status code after an operation failed and
+     * remember that this instance has failed
+     *
+     * output format: <failure>
+     *
+     * @param status     a more specific status code; other throwError() variants
+     *                   use STATUS_FATAL + sysync::LOCAL_STATUS_CODE, which is interpreted
+     *                   as a fatal local error
+     * @param action     a string describing what was attempted *and* how it failed
+     */
+    static void throwError(SyncMLStatus status, const string &failure);
+
     /**
      * throw an exception after an operation failed and
      * remember that this instance has failed
@@ -432,6 +516,14 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     bool getRemoteInitiated() {return m_remoteInitiated;}
     void setRemoteInitiated(bool remote) {m_remoteInitiated = remote;}
 
+    /**
+     * Read from stdin until end of stream.
+     *
+     * Default implementation reads from real stdin,
+     * D-Bus server implementation must ask client.
+     */
+    virtual void readStdin(string &content);
+
   protected:
     /** exchange active Synthesis engine */
     SharedEngine swapEngine(SharedEngine newengine) {
@@ -599,9 +691,8 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      *
      * The agent must be ready for use:
      * - HTTP specific settings must have been applied
-     * - the current SyncContect's transport_cb() must have been
-     *   installed via TransportAgent::setCallback(), with a suitable
-     *   timeout for the agent
+     * - the current SyncContext's timeout must have been
+     *   installed via TransportAgent::setTimeout()
      *
      * The default implementation instantiates one of the builtin
      * transport agents, depending on how it was compiled.
@@ -699,6 +790,12 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      */
     void initSources(SourceList &sourceList);
 
+    /**
+     * set m_localSync and m_localPeerContext
+     * @param config    config name of peer
+     */
+    void initLocalSync(const string &config);
+
     /**
      * called by SynthesDBPlugin in SyncEvolution_StartDataRead()
      */
@@ -728,7 +825,7 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      * Synthesis server textdb files, unique for each
      * peer
      */
-    string getSynthesisDatadir() { return getRootPath() + "/.synthesis"; }
+    string getSynthesisDatadir();
 
     /**
      * return true if "delayedabort" session variable is true
@@ -750,8 +847,12 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     ostream *m_out;
 
 public:
-    static bool transport_cb (void *data);
-
+    /**
+     * Returns the URL in the getSyncURL() list which is to be used
+     * for sync.  The long term goal is to pick the first URL which
+     * uses a currently available transport; right now it simply picks
+     * the first supported one.
+     */
     string getUsedSyncURL();
 };
 
