@@ -35,7 +35,7 @@
 SE_BEGIN_CXX
 
 class SyncSource;
-class SDKInterface;
+struct SDKInterface;
 
 /**
  * This set of parameters always has to be passed when constructing
@@ -45,15 +45,34 @@ struct SyncSourceParams {
     /**
      * @param    name        the name needed by SyncSource
      * @param    nodes       a set of config nodes to be used by this source
+     * @param    context     Additional non-source config settings.
+     *                       When running as part of a normal sync, these are the
+     *                       settings for the peer. When running in a local sync,
+     *                       these settings come from the "target-config" peer
+     *                       config inside the config context of the source.
+     *                       Testing uses "target-config@client-test". On the
+     *                       command line, this is the config chosen by the
+     *                       user, which may or may not have peer-specific settings!
+     * @param    contextName optional name of context in which the source is defined,
+     *                       needed to disambiguates "name" when sources from
+     *                       different contexts are active in a sync
      */
     SyncSourceParams(const string &name,
-                     const SyncSourceNodes &nodes = SyncSourceNodes()) :
+                     const SyncSourceNodes &nodes,
+                     const boost::shared_ptr<SyncConfig> &context,
+                     const string &contextName = "") :
         m_name(name),
-        m_nodes(nodes)
+        m_nodes(nodes),
+        m_context(context),
+        m_contextName(contextName)
     {}
 
+    std::string getDisplayName() const { return m_contextName.empty() ? m_name : m_contextName + "/" + m_name; }
+
     string m_name;
     SyncSourceNodes m_nodes;
+    boost::shared_ptr<SyncConfig> m_context;
+    string m_contextName;
 };
 
 /**
@@ -184,7 +203,7 @@ struct ClientTestConfig{
      * @param isSourceA true if the requested SyncSource is the first one accessing that
      *                  data, otherwise the second
      */
-    typedef TestingSyncSource *(*createsource_t)(ClientTest &client, int source, bool isSourceA);
+    typedef boost::function<TestingSyncSource *(ClientTest &, int, bool)> createsource_t;
 
     /**
      * Creates a sync source which references the primary database;
@@ -249,6 +268,16 @@ struct ClientTestConfig{
      */
     const char *itemType;
 
+    /**
+     * callback which is invoked with a specific item as paramter
+     * to do data type specific conversions before actually
+     * using the test item; default is a NOP function
+     *
+     * @param update     modify item content so that it can be
+     *                   used as an update of the old data
+     */
+    string (*mangleItem)(const char *data, bool update);
+
     /**
      * A very simple item that is inserted during basic tests. Ideally
      * it only contains properties supported by all servers.
@@ -347,7 +376,8 @@ struct ClientTestConfig{
      *                   this may depend on the current server that is being tested
      * @return error code, 0 for success
      */
-    int (*import)(ClientTest &client, TestingSyncSource &source, const char *file, std::string &realfile);
+    int (*import)(ClientTest &client, TestingSyncSource &source, const ClientTestConfig &config,
+                  const char *file, std::string &realfile);
 
     /**
      * a function which compares two files with items in the format used by "dump"
@@ -359,7 +389,21 @@ struct ClientTestConfig{
     bool (*compare)(ClientTest &client, const char *fileA, const char *fileB);
 
     /**
-     * a file with test cases in the format expected by import and compare
+     * A file with test cases in the format expected by import and compare.
+     * The file should contain data as supported by the local storage.
+     *
+     * It is used in "Source::*::testImport" test, which verifies that
+     * the backend can import and export that data.
+     *
+     * It is also used in "Sync::*::testItems", which verifies that
+     * the peer can store and export it. Often local extensions are
+     * not supported by peers. This can be handled in different ways:
+     * - Patch synccompare to ignore such changes on a per-peer basis.
+     * - Create a <testcases>.<peer>.tem file in the src/testcases
+     *   build directory where <testcases> is the string here ("eds_event.ics"),
+     *   and <peer> the value of CLIENT_TEST_SERVER ("funambol").
+     *   That file then will be used in testItems instead of the base
+     *   version. See the src/Makefile.am for rules that maintain such files.
      */
     const char *testcases;
 
@@ -382,6 +426,20 @@ struct ClientTestConfig{
     bool retrySync;
     bool suspendSync;
     bool resendSync;
+
+    /**
+     * Set this to test if the source supports preserving local data extensions.
+     * Uses the "testcases" data. See Sync::*::testExtensions.
+     *
+     * The function must modify a single item such that re-importing
+     * it locally will be seen as updating it. It is empty by default
+     * because not all backends necessarily pass this test.
+     *
+     * genericUpdate works for vCard and iCalendar by updating FN, N, resp. SUMMARY
+     * and can be used as implementation of update.
+     */
+    void (*update)(std::string &item);
+    void (*genericUpdate)(std::string &item);
 };
 
 /**
@@ -429,7 +487,7 @@ class RegisterSyncSourceTest
     /**
      * This call is invoked after setting up the config with default
      * values for the test cases selected via the constructor's
-     * testCaseName parameter (one of vcard21, vcard30, ical20, itodo20;
+     * testCaseName parameter (one of eds_contact, eds_contact, eds_event, eds_task;
      * see ClientTest in the Funambol client library for the current
      * list).
      *
@@ -448,7 +506,7 @@ class RegisterSyncSourceTest
      *                       ClientTest::getTestData() are already used for the initial
      *                       set of Evolution sync sources, for new sync sources
      *                       build a string by combining them with the sync source name
-     *                       (e.g., "sqlite_vcard30")
+     *                       (e.g., "sqlite_eds_contact")
      * @param testCaseName   a string recognized by ClientTest::getTestData() or an
      *                       empty string if there are no predefined test cases
      */
@@ -536,8 +594,20 @@ class SyncSourceBase : public Logger {
  public:
     virtual ~SyncSourceBase() {}
 
-    /** the unique name of the sync source (for example, "addressbook") */
-    virtual const char *getName() const { return "uninitialized SyncSourceBase"; }
+    /**
+     * the name of the sync source (for example, "addressbook"),
+     * unique in the context of its own configuration
+     **/
+    virtual std::string getName() const { return "uninitialized SyncSourceBase"; }
+
+    /**
+     * the name of the sync source as it should be displayed to users
+     * in debug messages; typically the same as getName(), but may
+     * also include a context ("@foobar/addressbook") to disambiguate
+     * the name when "addressbook" is used multiple times in a sync (as
+     * with local sync)
+     */
+    virtual std::string getDisplayName() const { return "uninitialized SyncSourceBase"; }
 
     /**
      * Convenience function, to be called inside a catch() block of
@@ -569,6 +639,17 @@ class SyncSourceBase : public Logger {
      */
     void throwError(const string &failure);
 
+    /**
+     * throw an exception with a specific status code after an operation failed and
+     * remember that this instance has failed
+     *
+     * output format: <source name>: <failure>
+     *
+     * @param status     a more specific status code; other throwError() variants use STATUS_FATAL
+     * @param action     a string describing what was attempted *and* how it failed
+     */
+    void throwError(SyncMLStatus status, const string &failure);
+
     /**
      * The Synthesis engine only counts items which are deleted by the
      * peer. Items deleted locally at the start of a
@@ -629,6 +710,7 @@ class SyncSourceBase : public Logger {
                           const char *function,
                           const char *format,
                           va_list args);
+    virtual bool isProcessSafe() const { return true; }
 
     /**
      * return Synthesis API pointer, if one currently is available
@@ -644,146 +726,6 @@ class SyncSourceBase : public Logger {
     virtual void enableServerMode() = 0;
     virtual bool serverModeEnabled() const = 0;
 
- protected:
-    struct SynthesisInfo {
-        /**
-         * name to use for MAKE/PARSETEXTWITHPROFILE,
-         * leave empty when acessing the field list directly
-         */
-        std::string m_profile;
-
-        /**
-         * the second parameter for MAKE/PARSETEXTWITHPROFILE
-         * which specifies a remote rule to be applied when
-         * converting to and from the backend
-         */
-        std::string m_backendRule;
-    
-        /** list of supported datatypes in "<use .../>" format */
-        std::string m_datatypes;
-
-        /** native datatype (see getNativeDatatypeName()) */
-        std::string m_native;
-
-        /** name of the field list used by the datatypes */
-        std::string m_fieldlist;
-
-        /**
-         * One or more Synthesis script statements, separated
-         * and terminated with a semicolon. Can be left empty.
-         *
-         * If not empty, then these statements are executed directly
-         * before converting the current item fields into
-         * a single string with MAKETEXTWITHPROFILE() in the sync source's
-         * <beforewritescript> (see SyncSourceBase::getDatastoreXML()).
-         *
-         * This value is currently only used by sync sources which
-         * set m_profile.
-         */
-        std::string m_beforeWriteScript;
-
-        /**
-         * Same as m_beforeWriteScript, but used directly after
-         * converting a string into fields with PARSETEXTWITHPROFILE()
-         * in <afterreadscript>.
-         */
-        std::string m_afterReadScript;
-    };
-
-    /**
-     * helper function for getDatastoreXML(): fill in information
-     * as necessary
-     *
-     * @retval fragments   the necessary definitions for the other
-     *                     return values have to be added here
-     */
-    virtual void getSynthesisInfo(SynthesisInfo &info,
-                                  XMLConfigFragments &fragments) = 0;
-
-    /**
-     * utility code: creates Synthesis <use datatype=...>
-     * statements, using the predefined vCard21/vCard30/vcalendar10/icalendar20
-     * types. Throws an error if no suitable result can be returned (empty or invalid type)
-     *
-     * @param type         the format specifier as used in SyncEvolution configs, with and without version
-     *                     (text/x-vcard:2.1, text/x-vcard, text/x-vcalendar, text/calendar, text/plain, ...);
-     *                     see SourceType::m_format
-     * @param forceFormat  if true, then don't allow alternative formats (like vCard 3.0 in addition to 2.1);
-     *                     see SourceType::m_force
-     * @return generated XML fragment
-     */
-    std::string getDataTypeSupport(const std::string &type,
-                                   bool forceFormat);
-};
-
-/**
- * SyncEvolution accesses all sources through this interface.
- *
- * Certain functionality is optional or can be implemented in
- * different ways. These methods are accessed through functors
- * (function objects) which may be unset. The expected usage is that
- * derived classes fill in the pieces that they provide by binding the
- * functors to normal methods. For example, TrackingSyncSource
- * provides a normal base class with pure virtual functions which have
- * to be provided by users of that class.
- *
- * Error reporting is done via the Log class.
- */
-class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, public SyncSourceReport
-{
- public:
-    SyncSource(const SyncSourceParams &params) :
-        SyncSourceConfig(params.m_name, params.m_nodes),
-        m_numDeleted(0),
-        m_forceSlowSync(false)
-        {
-        }
-    virtual ~SyncSource() {}
-
-    /**
-     * SyncSource implementations must register themselves here via
-     * RegisterSyncSource
-     */
-    static SourceRegistry &getSourceRegistry();
-
-    /**
-     * SyncSource tests are registered here by the constructor of
-     * RegisterSyncSourceTest
-     */
-    static TestRegistry &getTestRegistry();
-
-    struct Database {
-    Database(const string &name, const string &uri, bool isDefault = false) :
-        m_name( name ), m_uri( uri ), m_isDefault(isDefault) {}
-        string m_name;
-        string m_uri;
-        bool m_isDefault;
-    };
-    typedef vector<Database> Databases;
-    
-    /**
-     * returns a list of all know data sources for the kind of items
-     * supported by this sync source
-     */
-    virtual Databases getDatabases() = 0;
-
-    /**
-     * Actually opens the data source specified in the constructor,
-     * will throw the normal exceptions if that fails. Should
-     * not modify the state of the sync source.
-     *
-     * The expectation is that this call is fairly light-weight, but
-     * does enough checking to determine whether the source is
-     * usable. More expensive operations (like determining changes)
-     * should be done in the m_startDataRead callback (bound to
-     * beginSync() in some of the utility classes).
-     *
-     * In clients, it will be called for all sources before
-     * the sync starts. In servers, it is called for each source once
-     * the client asks for it, but not sooner.
-     */
-    virtual void open() = 0;
-
     /**
      * The optional operations.
      *
@@ -1007,7 +949,163 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
         boost::function<DeleteBlob_t> m_deleteBlob;
         /**@}*/
     };
-    const Operations &getOperations() { return m_operations; }
+
+    /**
+     * Read-only access to operations.
+     */
+    virtual const Operations &getOperations() = 0;
+
+ protected:
+    struct SynthesisInfo {
+        /**
+         * name to use for MAKE/PARSETEXTWITHPROFILE,
+         * leave empty when acessing the field list directly
+         */
+        std::string m_profile;
+
+        /**
+         * the second parameter for MAKE/PARSETEXTWITHPROFILE
+         * which specifies a remote rule to be applied when
+         * converting to and from the backend
+         */
+        std::string m_backendRule;
+    
+        /** list of supported datatypes in "<use .../>" format */
+        std::string m_datatypes;
+
+        /** native datatype (see getNativeDatatypeName()) */
+        std::string m_native;
+
+        /** name of the field list used by the datatypes */
+        std::string m_fieldlist;
+
+        /**
+         * One or more Synthesis script statements, separated
+         * and terminated with a semicolon. Can be left empty.
+         *
+         * If not empty, then these statements are executed directly
+         * before converting the current item fields into
+         * a single string with MAKETEXTWITHPROFILE() in the sync source's
+         * <beforewritescript> (see SyncSourceBase::getDatastoreXML()).
+         *
+         * This value is currently only used by sync sources which
+         * set m_profile.
+         */
+        std::string m_beforeWriteScript;
+
+        /**
+         * Same as m_beforeWriteScript, but used directly after
+         * converting a string into fields with PARSETEXTWITHPROFILE()
+         * in <afterreadscript>.
+         */
+        std::string m_afterReadScript;
+
+        /**
+         * Arbitrary configuration options, can override the ones above
+         * because they are added to the <datastore></datastore>
+         * XML configuration directly before the closing element.
+         *
+         * One example is adding <updateallfields>: this is necessary
+         * in backends which depend on getting complete items (= for example,
+         * vCard 3.0 strings) from the engine.
+         */
+        std::string m_datastoreOptions;
+    };
+
+    /**
+     * helper function for getDatastoreXML(): fill in information
+     * as necessary
+     *
+     * @retval fragments   the necessary definitions for the other
+     *                     return values have to be added here
+     */
+    virtual void getSynthesisInfo(SynthesisInfo &info,
+                                  XMLConfigFragments &fragments) = 0;
+
+    /**
+     * utility code: creates Synthesis <use datatype=...>
+     * statements, using the predefined vCard21/vCard30/vcalendar10/icalendar20
+     * types. Throws an error if no suitable result can be returned (empty or invalid type)
+     *
+     * @param type         the format specifier as used in SyncEvolution configs, with and without version
+     *                     (text/x-vcard:2.1, text/x-vcard, text/x-vcalendar, text/calendar, text/plain, ...);
+     *                     see SourceType::m_format
+     * @param forceFormat  if true, then don't allow alternative formats (like vCard 3.0 in addition to 2.1);
+     *                     see SourceType::m_force
+     * @return generated XML fragment
+     */
+    std::string getDataTypeSupport(const std::string &type,
+                                   bool forceFormat);
+};
+
+/**
+ * SyncEvolution accesses all sources through this interface.
+ *
+ * Certain functionality is optional or can be implemented in
+ * different ways. These methods are accessed through functors
+ * (function objects) which may be unset. The expected usage is that
+ * derived classes fill in the pieces that they provide by binding the
+ * functors to normal methods. For example, TrackingSyncSource
+ * provides a normal base class with pure virtual functions which have
+ * to be provided by users of that class.
+ *
+ * Error reporting is done via the Log class.
+ */
+class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, public SyncSourceReport
+{
+ public:
+    SyncSource(const SyncSourceParams &params);
+    virtual ~SyncSource() {}
+
+    /**
+     * SyncSource implementations must register themselves here via
+     * RegisterSyncSource
+     */
+    static SourceRegistry &getSourceRegistry();
+
+    /**
+     * SyncSource tests are registered here by the constructor of
+     * RegisterSyncSourceTest
+     */
+    static TestRegistry &getTestRegistry();
+
+    struct Database {
+    Database(const string &name, const string &uri, bool isDefault = false) :
+        m_name( name ), m_uri( uri ), m_isDefault(isDefault) {}
+        string m_name;
+        string m_uri;
+        bool m_isDefault;
+    };
+    typedef vector<Database> Databases;
+    
+    /**
+     * returns a list of all know data sources for the kind of items
+     * supported by this sync source
+     */
+    virtual Databases getDatabases() = 0;
+
+    /**
+     * Actually opens the data source specified in the constructor,
+     * will throw the normal exceptions if that fails. Should
+     * not modify the state of the sync source.
+     *
+     * The expectation is that this call is fairly light-weight, but
+     * does enough checking to determine whether the source is
+     * usable. More expensive operations (like determining changes)
+     * should be done in the m_startDataRead callback (bound to
+     * beginSync() in some of the utility classes).
+     *
+     * In clients, it will be called for all sources before
+     * the sync starts. In servers, it is called for each source once
+     * the client asks for it, but not sooner.
+     */
+    virtual void open() = 0;
+
+    /**
+     * Read-only access to operations.  Derived classes can modify
+     * them via m_operations.
+     */
+    virtual const Operations &getOperations() { return m_operations; }
 
     /**
      * outside users of the source are only allowed to add callbacks,
@@ -1086,10 +1184,11 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      * Mime type a backend communicates with the remote peer by default,
      * this is used to alert the remote peer in SAN during server alerted sync.
      */
-    virtual const char *getPeerMimeType() const =0;
+    virtual std::string getPeerMimeType() const =0;
 
     /* implementation of SyncSourceBase */
-    virtual const char * getName() const { return SyncSourceConfig::getName(); }
+    virtual std::string getName() const { return SyncSourceConfig::getName(); }
+    virtual std::string getDisplayName() const { return m_name.c_str(); }
     virtual long getNumDeleted() const { return m_numDeleted; }
     virtual void setNumDeleted(long num) { m_numDeleted = num; }
     virtual void incrementNumDeleted() { m_numDeleted++; }
@@ -1127,6 +1226,9 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      * the engine is running.
      */
     std::vector<sysync::SDK_InterfaceType *> m_synthesisAPI;
+
+    /** actual name of the source */
+    std::string m_name;
 };
 
 /**
@@ -1138,8 +1240,8 @@ class DummySyncSource : public SyncSource
     DummySyncSource(const SyncSourceParams &params) :
        SyncSource(params) {}
 
-    DummySyncSource(const std::string &name) :
-       SyncSource(SyncSourceParams(name)) {}
+     DummySyncSource(const std::string &name, const std::string &contextName) :
+       SyncSource(SyncSourceParams(name, SyncSourceNodes(), boost::shared_ptr<SyncConfig>(), contextName)) {}
 
     virtual Databases getDatabases() { return Databases(); }
     virtual void open() {}
@@ -1148,7 +1250,7 @@ class DummySyncSource : public SyncSource
                                   XMLConfigFragments &fragments) {}
     virtual void enableServerMode() {}
     virtual bool serverModeEnabled() const { return false; }
-    virtual const char *getPeerMimeType() const {return "";} 
+    virtual std::string getPeerMimeType() const {return "";} 
 };
 
 /**
@@ -1263,7 +1365,7 @@ class SyncSourceChanges : virtual public SyncSourceBase {
      * and does not distinguish between added and updated items, so
      * for syncing, adding DELETED items is optional and all items
      * which are different from the last sync can be added as
-     * UPDATED. The client-test program expects that the information
+     * UPDATED. The client-test program expects that the informationb
      * is provided precisely.
      *
      * @return true if the luid was already listed
@@ -1385,13 +1487,13 @@ class SyncSourceSerialize : virtual public SyncSourceBase, virtual public SyncSo
      * Returns the preferred mime type of the items handled by the sync source.
      * Example: "text/x-vcard"
      */
-    virtual const char *getMimeType() const = 0;
+    virtual std::string getMimeType() const = 0;
 
     /**
      * Returns the version of the mime type used by client.
      * Example: "2.1"
      */
-    virtual const char *getMimeVersion() const = 0;
+    virtual std::string getMimeVersion() const = 0;
 
     /**
      * returns the backend selection and configuration
@@ -1450,6 +1552,78 @@ class SyncSourceSerialize : virtual public SyncSourceBase, virtual public SyncSo
     sysync::TSyError insertItemAsKey(sysync::KeyH aItemKey, sysync::cItemID aID, sysync::ItemID newID);
 };
 
+/**
+ * Mapping from Hash() value to file.
+ * Used by SyncSourceRevisions, but may be of use for
+ * other backup implementations.
+ */
+class ItemCache
+{
+public:
+#ifdef USE_SHA256
+    typedef std::string Hash_t;
+    Hash_t hashFunc(const std::string &data) { return SHA_256(data); }
+#else
+    typedef unsigned long Hash_t;
+    Hash_t hashFunc(const std::string &data) { return Hash(data); }
+#endif
+    typedef unsigned long Counter_t;
+
+    /** mark the algorithm used for the hash via different suffices */
+    static const char *m_hashSuffix;
+
+    /**
+     * Collect information about stored hashes. Provides
+     * access to file name via hash.
+     *
+     * If no hashes were written (as in an old SyncEvoltion
+     * version), we could read the files to recreate the
+     * hashes. This is not done because it won't occur
+     * often enough.
+     *
+     * Hashes are also not verified. Users should better
+     * not edit them or file contents...
+     *
+     * @param oldBackup     existing backup to read; may be empty
+     * @param newBackup     new backup to be created
+     * @param legacy        legacy mode includes a bug
+     *                      which cannot be fixed without breaking on-disk format
+     */
+    void init(const SyncSource::Operations::ConstBackupInfo &oldBackup,
+              const SyncSource::Operations::BackupInfo &newBackup,
+              bool legacy);
+
+    /**
+     * create file name for a specific hash, empty if no such hash
+     */
+    string getFilename(Hash_t hash);
+
+    /**
+     * add a new item, reusing old one if possible
+     *
+     * @param item       new item data
+     * @param uid        its unique ID
+     * @param rev        revision string
+     */
+    void backupItem(const std::string &item,
+                    const std::string &uid,
+                    const std::string &rev);
+
+    /** to be called after init() and all backupItem() calls */
+    void finalize(BackupReport &report);
+
+    /** can be used to restart creating the backup after an intermediate failure */
+    void reset();
+
+private:
+    typedef std::map<Hash_t, Counter_t> Map_t;
+    Map_t m_hash2counter;
+    string m_dirname;
+    SyncSource::Operations::BackupInfo m_backup;
+    bool m_legacy;
+    unsigned long m_counter;
+};
+
 /**
  * Implements change tracking based on a "revision" string, a string
  * which is guaranteed to change automatically each time an item is
@@ -1495,8 +1669,8 @@ class SyncSourceRevisions : virtual public SyncSourceChanges, virtual public Syn
     typedef map<string, string> RevisionMap_t;
 
     /**
-     * fills the complete mapping from UID to revision string of all
-     * currently existing items
+     * Fills the complete mapping from UID to revision string of all
+     * currently existing items.
      *
      * Usually both UID and revision string must be non-empty. The
      * only exception is a refresh-from-client: in that case the
@@ -1507,11 +1681,80 @@ class SyncSourceRevisions : virtual public SyncSourceChanges, virtual public Syn
      * a non-empty string is necessary and none was provided.
      *
      * This call is typically only invoked only once during the
-     * lifetime of a source. The result returned in that invocation is
+     * lifetime of a source, at the time when detectChanges() needs
+     * the information. The result returned in that invocation is
      * used throught the session.
+     *
+     * When detectChanges() is called with CHANGES_NONE, listAllItems()
+     * is avoided. Instead the cached information is used. Sources
+     * may need to know that information, so in this case setAllItems()
+     * is called as part of detectChanges().
      */
     virtual void listAllItems(RevisionMap_t &revisions) = 0;
 
+    /**
+     * Called by SyncSourceRevisions::detectChanges() to tell
+     * the derived class about the cached information if (and only
+     * if) listAllItems() and updateAllItems() were not called. The derived class
+     * might not need this information, so the default implementation
+     * simply ignores.
+     *
+     * A more complex API could have been defined to only prepare the
+     * information when needed, but that seemed unnecessarily complex.
+     */
+    virtual void setAllItems(const RevisionMap_t &revisions) {}
+
+    /**
+     * updates the revision map to reflect the current state
+     *
+     * May be called instead of listAllItems() if the caller has
+     * a valid list to start from. If the implementor
+     * cannot update the list, it must start from scratch by
+     * reseting the list and calling listAllItems(). The default
+     * implementation of this method does that.
+     */
+    virtual void updateAllItems(SyncSourceRevisions::RevisionMap_t &revisions) {
+        revisions.clear();
+        listAllItems(revisions);
+    }
+
+    /**
+     * Tells detectChanges() how to do its job.
+     */
+    enum ChangeMode {
+        /**
+         * Call listAllItems() and use the list of previous items
+         * to calculate changes.
+         */
+        CHANGES_FULL,
+
+        /**
+         * Don't rely on previous information. Will call
+         * listAllItems() and generate a full list of items based on
+         * the result.
+         *
+         * TODO: Added/updated/deleted information is still getting
+         * calculated based on the previous items although it is not
+         * needed. In other words, CHANGES_SLOW == CHANGES_FULL at the
+         * moment. Once we are sure that slow sync detection works,
+         * calculating changes in this mode can be removed.
+         */
+        CHANGES_SLOW,
+
+        /**
+         * Caller has already determined that a) no items have changed
+         * and that b) the list of previous items is valid. For example,
+         * some backends have a way of getting a revision string for
+         * the whole database and can compare that against the value
+         * from the end of the previous sync.
+         *
+         * In this mode, listAllItems() doesn't have to be called.
+         * A list of all items will be created, with no items marked
+         * as added/updated/deleted.
+         */
+        CHANGES_NONE
+    };
+
     /**
      * calculate changes, call when sync source is ready for
      * listAllItems() and before changes are needed
@@ -1521,8 +1764,12 @@ class SyncSourceRevisions : virtual public SyncSourceChanges, virtual public Syn
      * the caller.
      *
      * @param trackingNode     a config node for exclusive use by this class
+     * @param mode             determines how changes are detected; if unsure,
+     *                         use CHANGES_FULL, which will always produce
+     *                         the required information, albeit more slowly
+     *                         than the other modes
      */
-    void detectChanges(ConfigNode &trackingNode);
+    void detectChanges(ConfigNode &trackingNode, ChangeMode mode);
 
     /**
      * record that an item was added or updated
@@ -1713,7 +1960,20 @@ class SyncSourceBlob : public virtual SyncSourceBase
                               void **aBlkPtr, size_t *aBlkSize,
                               size_t *aTotSize,
                               bool aFirst, bool *aLast) {
-        return m_blob.ReadBlob(aID, aBlobID, aBlkPtr, aBlkSize, aTotSize, aFirst, aLast);
+        // Translate between sysync::memSize and size_t, which
+        // is different on s390 (or at least the compiler complains...).
+        sysync::memSize blksize, totsize;
+        sysync::TSyError err = m_blob.ReadBlob(aID, aBlobID, aBlkPtr,
+                                               aBlkSize ? &blksize : NULL,
+                                               aTotSize ? &totsize : NULL,
+                                               aFirst, aLast);
+        if (aBlkSize) {
+            *aBlkSize = blksize;
+        }
+        if (aTotSize) {
+            *aTotSize = totsize;
+        }
+        return err;
     }
     sysync::TSyError writeBlob(sysync::cItemID aID, const char *aBlobID,
                                void *aBlkPtr, size_t aBlkSize,
