@@ -52,7 +52,7 @@ static SyncSource *MoC(CContext mContext) { return (SyncSource *)mContext; }
 /**
  * looks up datasource and uses pointer to it as context
  *
- * @param mContextName   name of previously instantiated SyncSource
+ * @param mContextName   name of previously instantiated SyncSource, "" when used as session module
  * @retval mContext      the corresponding SyncSource
  */
 extern "C"
@@ -62,17 +62,20 @@ TSyError SyncEvolution_Module_CreateContext( CContext *mContext, cAppCharP   mod
                                              DB_Callback mCB )
 {
     TSyError err = LOCERR_WRONGUSAGE;
-    SyncSource *source = SyncContext::findSource(mContextName);
-    if (source) {
-        source->pushSynthesisAPI(mCB);
-        *mContext = (CContext)source;
+    if (!mContextName[0]) {
+        *mContext = NULL;
         err = LOCERR_OK;
+    } else {
+        SyncSource *source = SyncContext::findSource(mContextName);
+        if (source) {
+            source->pushSynthesisAPI(mCB);
+            *mContext = (CContext)source;
+            err = LOCERR_OK;
+        }
     }
 
-    DEBUG_DB(mCB, MyDB,  Mo_CC, "'%s%s%s' (%s) => %d",
-             moduleName, subName[0] ? "/" : "", subName, 
-             mContextName,
-             err);
+    SE_LOG_DEBUG(NULL, NULL, "CreateContext %s/%s/%s => %d",
+                 moduleName, subName, mContextName, err);
     return err;
 }
 
@@ -87,7 +90,7 @@ CVersion SyncEvolution_Module_Version(CContext mContext)
     CVersion v = Plugin_Version(BuildNumber);
   
     if (mContext) {
-        DEBUG_DB(MoC(mContext)->getSynthesisAPI(), MyDB,Mo_Ve, "%08X", v);
+        SE_LOG_DEBUG(NULL, NULL, "Module_Version = %08lx", (long)v);
     }
     return v;
 }
@@ -98,6 +101,8 @@ CVersion SyncEvolution_Module_Version(CContext mContext)
 extern "C"
 TSyError SyncEvolution_Module_Capabilities( CContext mContext, appCharP *mCapabilities )
 {
+    SyncSource *source  = MoC(mContext);
+
     std::stringstream s;
     s << MyPlatform() << "\n"
       << DLL_Info << "\n"
@@ -107,9 +112,17 @@ TSyError SyncEvolution_Module_Capabilities( CContext mContext, appCharP *mCapabi
       << Plugin_DS_Data_Str << ":no\n"
       << Plugin_DS_Data_Key << ":yes\n"
       << CA_ItemAsKey << ":yes\n"
-      << Plugin_DS_Blob << ":no\n";
+      << Plugin_DS_Blob <<
+        ((source && source->getOperations().m_readBlob) ?
+         ":yes\n" :
+         ":no\n");
+
+    if (source && source->getOperations().m_loadAdminData) {
+        s << Plugin_DS_Admin << ":yes\n";
+    }
+
     *mCapabilities= StrAlloc(s.str().c_str());
-    DEBUG_DB(MoC(mContext)->getSynthesisAPI(), MyDB, Mo_Ca, "'%s'", *mCapabilities);
+    SE_LOG_DEBUG(NULL, NULL, "Module_Capabilities:\n%s", *mCapabilities);
     return LOCERR_OK;
 } /* Module_Capabilities */
 
@@ -120,9 +133,8 @@ TSyError SyncEvolution_Module_PluginParams( CContext mContext,
                                             cAppCharP mConfigParams, CVersion engineVersion )
 {
     SyncSource *source  = MoC(mContext);
-    DEBUG_DB(source->getSynthesisAPI(), MyDB, Mo_PP, " Engine=%08X", engineVersion);
-    DEBUG_DB(source->getSynthesisAPI(), MyDB, Mo_PP, "'%s'",         mConfigParams );
-  
+    SE_LOG_DEBUG(source, NULL, "Module_PluginParams\n Engine=%08lX\n %s",
+                 (long)engineVersion, mConfigParams);
     /*return LOCERR_CFGPARSE;*/ /* if there are unsupported params */
     return LOCERR_OK;
 } /* Module_PluginParams */
@@ -133,8 +145,6 @@ TSyError SyncEvolution_Module_PluginParams( CContext mContext,
 extern "C"
 void SyncEvolution_Module_DisposeObj( CContext mContext, void* memory )
 {
-    DEBUG_Exotic_DB(MoC(mContext)->getSynthesisAPI(), MyDB, Mo_DO,
-                    "free at %08X '%s'", memory, memory);
     StrDispose(memory);
 }
 
@@ -142,51 +152,32 @@ extern "C"
 TSyError SyncEvolution_Module_DeleteContext( CContext mContext )
 {
     SyncSource  *source = MoC(mContext);
-    DEBUG_DB(source->getSynthesisAPI(), MyDB, Mo_DC, "'%s'", source->getName());
-    source->popSynthesisAPI();
+    SE_LOG_DEBUG(NULL, NULL, "Module_DeleteContext %s",
+                 source ? source->getName().c_str() : "'session'");
+    if (source) {
+        source->popSynthesisAPI();
+    }
     return LOCERR_OK;
 }
 
 
-
-
-/* ---------------------- session handling --------------------- */
-/* this is an example, how a context could be structured */
-class SessionContext {
-public:
-    SessionContext() { fCB= NULL; }
-    
-    int         fID; /* a reference number. */
-    DB_Callback fCB; /* callback structure  */
-    int      fPMode; /* The login password mode */
-};
-
-/* <sContext> will be casted to the SessionContext* structure */
-static SessionContext* SeC( CContext sContext ) { return (SessionContext*)sContext; }
+/* <sContext> will be casted to the SyncContext* structure */
+static SyncContext* SeC( CContext sContext ) { return (SyncContext*)sContext; }
 
 
 
-/* Create a context for a new session */ 
+/* Create a context for a new session. Maps to the existing SyncContext. */ 
 extern "C"
 TSyError SyncEvolution_Session_CreateContext( CContext *sContext, cAppCharP sessionName, DB_Callback sCB )
 { 
-  SessionContext* sc;
-  
-/*return DB_Error;*/ /* added for test */
-  
-  sc= new SessionContext;
-
-  if (sc==NULL) return DB_Full;
-
-  /**** CAN BE ADAPTED BY USER ****/
-            sc->fID= 333; /* as an example */
-            sc->fCB= sCB;
-            sc->fPMode= Password_ClrText_IN;        /* take this mode ... */
-         /* sc->fPMode= Password_ClrText_OUT; */    /* ... or this        */
-  DEBUG_DB( sc->fCB, MyDB,Se_CC, "%d '%s'", sc->fID,sessionName );
-  
-  *sContext= (CContext)sc; /* return the created context structure */
-  return LOCERR_OK;
+    *sContext = (CContext)SyncContext::findContext(sessionName);
+    SE_LOG_DEBUG(NULL, NULL, "Session_CreateContext '%s' %s", 
+                 sessionName, *sContext ? "found" : "not found");
+    if (*sContext) {
+        return LOCERR_OK;
+    } else {
+        return DB_NotFound;
+    }
 } /* Session_CreateContext */
 
 
@@ -198,11 +189,11 @@ TSyError SyncEvolution_Session_AdaptItem( CContext sContext, appCharP *sItemData
                                           appCharP *sLocalVars, 
                                           uInt32  sIdentifier ) 
 { 
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,"Session_AdaptItem", "'%s' '%s' '%s' id=%d", 
-                                *sItemData1,*sItemData2,*sLocalVars, sIdentifier );
-  return LOCERR_OK;
+    /**** CAN BE ADAPTED BY USER ****/ 
+    // SyncContext* sc= SeC( sContext );
+    SE_LOG_DEBUG(NULL, NULL, "Session_AdaptItem '%s' '%s' '%s' id=%d", 
+                 *sItemData1,*sItemData2,*sLocalVars, sIdentifier);
+    return LOCERR_OK;
 } /* Session_AdaptItem */
 
 
@@ -213,13 +204,29 @@ TSyError SyncEvolution_Session_CheckDevice( CContext sContext,
                                             cAppCharP aDeviceID, appCharP *sDevKey,
                                             appCharP *nonce )
 {
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  
-  *sDevKey= StrAlloc( aDeviceID  );
-  *nonce  = StrAlloc( "xyz_last" );
-  DEBUG_DB( sc->fCB, MyDB,Se_CD, "%d dev='%s' nonce='%s'", sc->fID, *sDevKey,*nonce );
-  return LOCERR_OK;
+    SyncContext *sc = SeC(sContext);
+    if (!sc) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = LOCERR_OK;
+
+    sc->setSyncDeviceID(aDeviceID);
+    string id = sc->getRemoteDevID();
+    if (id.empty()) {
+        sc->setRemoteDevID(aDeviceID);
+        sc->flush();
+    } else if (id != aDeviceID) {
+        // We are using the wrong configuration?! Refuse to continue.
+        SE_LOG_ERROR(NULL, NULL, "remote device ID '%s' in config does not match the one from the peer '%s' - incorrect configuration?!",
+                     id.c_str(), aDeviceID);
+        res = DB_Forbidden;
+    }
+
+    *sDevKey= StrAlloc(aDeviceID);
+    *nonce = StrAlloc(sc->getNonce().c_str());
+    SE_LOG_DEBUG(NULL, NULL, "Session_CheckDevice dev='%s' nonce='%s' res=%d",
+                 *sDevKey, *nonce, res);
+    return res;
 } /* Session_CheckDevice */
 
 
@@ -230,11 +237,7 @@ TSyError SyncEvolution_Session_CheckDevice( CContext sContext,
 extern "C"
 TSyError SyncEvolution_Session_GetNonce( CContext sContext, appCharP *nonce )
 {
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_GN, "%d (not supported)", sc->fID );
-  *nonce= NULL;
-  return DB_NotFound;
+    return DB_NotFound;
 } /* Session_GetNonce */
 
  
@@ -245,10 +248,15 @@ TSyError SyncEvolution_Session_GetNonce( CContext sContext, appCharP *nonce )
 extern "C"
 TSyError SyncEvolution_Session_SaveNonce( CContext sContext, cAppCharP nonce )
 {
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_SN, "%d nonce='%s'", sc->fID, nonce );
-  return LOCERR_OK;
+    SyncContext *sc = SeC(sContext);
+    if (!sc) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(NULL, NULL, "Session_SaveNonce nonce='%s'",
+                 nonce);
+    sc->setNonce(nonce);
+    sc->flush();
+    return LOCERR_OK;
 } /* Session_SaveNonce */
 
 
@@ -257,10 +265,15 @@ TSyError SyncEvolution_Session_SaveNonce( CContext sContext, cAppCharP nonce )
 extern "C"
 TSyError SyncEvolution_Session_SaveDeviceInfo( CContext sContext, cAppCharP aDeviceInfo )
 {
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_SD, "%d info='%s'", sc->fID, aDeviceInfo );
-  return LOCERR_OK;
+    SyncContext *sc = SeC(sContext);
+    if (!sc) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(NULL, NULL, "Session_SaveDeviceInfo info='%s'",
+                 aDeviceInfo );
+    sc->setDeviceData(aDeviceInfo);
+    sc->flush();
+    return LOCERR_OK;
 } /* Session_SaveDeviceInfo */
 
 
@@ -269,11 +282,7 @@ TSyError SyncEvolution_Session_SaveDeviceInfo( CContext sContext, cAppCharP aDev
 extern "C"
 TSyError SyncEvolution_Session_GetDBTime( CContext sContext, appCharP *currentDBTime )
 { 
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_GT, "%d", sc->fID );
-  *currentDBTime= NULL;
-  return DB_NotFound;
+    return DB_NotFound;
 } /* Session_GetDBTime */
 
 
@@ -286,10 +295,7 @@ TSyError SyncEvolution_Session_GetDBTime( CContext sContext, appCharP *currentDB
 extern "C"
 sInt32 SyncEvolution_Session_PasswordMode( CContext sContext )
 {
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_PM, "%d mode=%d", sc->fID, sc->fPMode );
-  return          sc->fPMode;
+    return Password_ClrText_OUT;
 } /* Session_PasswordMode */
 
 
@@ -299,25 +305,25 @@ extern "C"
 TSyError SyncEvolution_Session_Login( CContext sContext, cAppCharP sUsername, appCharP *sPassword,
                                       appCharP *sUsrKey )
 { 
-  /**** CAN BE ADAPTED BY USER ****/
-  SessionContext* sc= SeC( sContext );
-  TSyError       err= DB_Forbidden; /* default */
-  
-  /* different modes, choose one for the plugin */
-  if (sc->fPMode==Password_ClrText_IN) {
-    if (strcmp(  sUsername,"super" )==0 &&
-        strcmp( *sPassword,"user"  )==0) { *sUsrKey  = StrAlloc( "1234" ); err= LOCERR_OK; }
-  }
-  else {       /* Password will be returned */
-    if (strcmp(  sUsername,"super" )==0) { *sPassword= StrAlloc( "user" );
-                                           *sUsrKey  = StrAlloc( "1234" ); err= LOCERR_OK; }
-  } /* if */
+    SyncContext *sc = SeC(sContext);
+    if (!sc) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    string user = sc->getSyncUsername();
+    string password = sc->getSyncPassword();
 
-  if (err) { DEBUG_DB( sc->fCB, MyDB,Se_LI, "%d usr='%s' err=%d",
-                                             sc->fID,sUsername, err ); }
-  else       DEBUG_DB( sc->fCB, MyDB,Se_LI, "%d usr='%s' pwd='%s' => key='%s'", 
-                                             sc->fID,sUsername,*sPassword, *sUsrKey );
-  return err;
+    if (user.empty() && password.empty()) {
+        // nothing to check, accept peer
+        res = LOCERR_OK;
+    } else if (user == sUsername) {
+        *sPassword=StrAlloc(password.c_str());
+        res = LOCERR_OK;
+    }
+
+    SE_LOG_DEBUG(NULL, NULL, "Session_Login usr='%s' expected user='%s' res=%d",
+                 sUsername, user.c_str(), res);
+    return res;
 } /* Session_Login */
 
 
@@ -326,10 +332,7 @@ TSyError SyncEvolution_Session_Login( CContext sContext, cAppCharP sUsername, ap
 extern "C"
 TSyError SyncEvolution_Session_Logout( CContext sContext )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_LO, "%d",sc->fID );
-  return LOCERR_OK;
+    return LOCERR_OK;
 } /* Session_Logout */
 
 
@@ -337,10 +340,6 @@ TSyError SyncEvolution_Session_Logout( CContext sContext )
 extern "C"
 void SyncEvolution_Session_DisposeObj( CContext sContext, void* memory )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext*  sc= SeC( sContext );
-  DEBUG_Exotic_DB( sc->fCB, MyDB,Se_DO, "%d free at %08X '%s'",
-                   sc->fID, memory,memory );
   StrDispose              ( memory );
 } /* Session_DisposeObj */
 
@@ -350,9 +349,6 @@ void SyncEvolution_Session_DisposeObj( CContext sContext, void* memory )
 extern "C"
 void SyncEvolution_Session_ThreadMayChangeNow( CContext sContext )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext*  sc= SeC( sContext );
-  DEBUG_Exotic_DB( sc->fCB, MyDB,Se_TC, "%d",sc->fID );
 } /* Session_ThreadMayChangeNow */
 
 
@@ -363,10 +359,6 @@ void SyncEvolution_Session_ThreadMayChangeNow( CContext sContext )
 extern "C"
 void SyncEvolution_Session_DispItems( CContext sContext, bool allFields, cAppCharP specificItem )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_DI, "%d %d '%s'",
-                  sc->fID, allFields,specificItem );
 } /* Session_DispItems */
 
 
@@ -375,13 +367,7 @@ void SyncEvolution_Session_DispItems( CContext sContext, bool allFields, cAppCha
 extern "C"
 TSyError SyncEvolution_Session_DeleteContext( CContext sContext ) 
 { 
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SessionContext* sc= SeC( sContext );
-  DEBUG_DB      ( sc->fCB, MyDB,Se_DC, "%d",sc->fID );
-
-  delete sc;
-
-  return LOCERR_OK;
+    return LOCERR_OK;
 } /* Session_DeleteContext */
 
 
@@ -413,8 +399,8 @@ TSyError SyncEvolution_CreateContext( CContext *aContext, cAppCharP aContextName
         *aContext = (CContext)source;
         err = LOCERR_OK;
     }
-    DEBUG_DB( aCB, MyDB,Da_CC, "'%s' dev='%s' usr='%s'",
-              aContextName, sDevKey, sUsrKey ); 
+    SE_LOG_DEBUG(source, NULL, "'%s' dev='%s' usr='%s' err=%d",
+                 aContextName, sDevKey, sUsrKey, err);
     return err;
 }
 
@@ -424,7 +410,10 @@ extern "C"
 uInt32 SyncEvolution_ContextSupport( CContext aContext, cAppCharP aContextRules ) 
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_CS, "%s '%s'", source->getName(),aContextRules );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(source, NULL, "ContextSupport %s", aContextRules);
     return 0;
 }
 
@@ -435,7 +424,10 @@ uInt32 SyncEvolution_FilterSupport( CContext aContext, cAppCharP aFilterRules )
 {
   /**** CAN BE ADAPTED BY USER ****/ 
   SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_FS, "%s '%s'", source->getName(),aFilterRules );
+  if (!source) {
+      return LOCERR_WRONGUSAGE;
+  }
+  SE_LOG_DEBUG(source, NULL, "FilterSupport %s", aFilterRules);
   return 0;
 } /* FilterSupport */
 
@@ -446,12 +438,22 @@ extern "C"
 TSyError SyncEvolution_LoadAdminData( CContext aContext, cAppCharP aLocDB,
                                       cAppCharP aRemDB, appCharP *adminData )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_LA, "%s '%s' '%s'",
-            source->getName(), aLocDB, aRemDB );
-  *adminData= NULL;
-  return DB_Forbidden; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    try {
+        if (source->getOperations().m_loadAdminData) {
+            res = source->getOperations().m_loadAdminData(aLocDB, aRemDB, adminData);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "LoadAdminData '%s' '%s', '%s' res=%d",
+                 aLocDB, aRemDB, *adminData ? *adminData : "", res);
+    return res;
 } /* LoadAdminData */
 
 
@@ -459,10 +461,21 @@ TSyError SyncEvolution_LoadAdminData( CContext aContext, cAppCharP aLocDB,
 extern "C"
 TSyError SyncEvolution_SaveAdminData( CContext aContext, cAppCharP adminData )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_SA, "%s '%s'", source->getName(), adminData );
-  return DB_Forbidden; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    try {
+        if (source->getOperations().m_saveAdminData) {
+            res = source->getOperations().m_saveAdminData(adminData);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "SaveAdminData '%s' res=%d", adminData, res);
+    return res;
 } /* SaveAdminData */
 
 
@@ -470,10 +483,27 @@ TSyError SyncEvolution_SaveAdminData( CContext aContext, cAppCharP adminData )
 extern "C"
 bool SyncEvolution_ReadNextMapItem( CContext aContext, MapID mID, bool aFirst )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_RM, "%s %08X first=%d (EOF)", source->getName(), mID, aFirst );
-  return false; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    bool res = false;
+    try {
+        if (source->getOperations().m_readNextMapItem) {
+            res = source->getOperations().m_readNextMapItem(mID, aFirst);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "ReadNextMapItem '%s' + %x = '%s' + %d first=%s res=%d",
+                 res ? NullPtrCheck(mID->localID) : "(none)",
+                 res ? mID->ident : 0,
+                 res ? NullPtrCheck(mID->remoteID) : "(none)",
+                 res ? mID->flags : 0,
+                 aFirst ? "yes" : "no",
+                 res);
+    return res;
 } /* ReadNextMapItem */
 
 
@@ -481,11 +511,24 @@ bool SyncEvolution_ReadNextMapItem( CContext aContext, MapID mID, bool aFirst )
 extern "C"
 TSyError SyncEvolution_InsertMapItem( CContext aContext, cMapID mID )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_IM, "%s %08X: '%s' '%s' %04X %d", 
-                                  source->getName(), mID, mID->localID, mID->remoteID, mID->flags, mID->ident );
-  return DB_Forbidden; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    try {
+        if (source->getOperations().m_insertMapItem) {
+            res = source->getOperations().m_insertMapItem(mID);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "InsertMapItem '%s' + %x = '%s' + %x res=%d", 
+                 NullPtrCheck(mID->localID), mID->ident,
+                 NullPtrCheck(mID->remoteID), mID->flags,
+                 res);
+    return res;
 } /* InsertMapItem */
 
 
@@ -493,11 +536,25 @@ TSyError SyncEvolution_InsertMapItem( CContext aContext, cMapID mID )
 extern "C"
 TSyError SyncEvolution_UpdateMapItem( CContext aContext, cMapID mID )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_UM, "%s %08X: '%s' '%s' %04X %d", 
-                                  source->getName(), mID, mID->localID, mID->remoteID, mID->flags, mID->ident );
-  return DB_Forbidden; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    try {
+        if (source->getOperations().m_updateMapItem) {
+            res = source->getOperations().m_updateMapItem(mID);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "UpdateMapItem '%s' + %x = '%s' + %x, res=%d", 
+                 mID->localID, mID->ident,
+                 mID->remoteID, mID->flags,
+                 res);
+
+    return res;
 } /* UpdateMapItem */
 
 
@@ -505,11 +562,24 @@ TSyError SyncEvolution_UpdateMapItem( CContext aContext, cMapID mID )
 extern "C"
 TSyError SyncEvolution_DeleteMapItem( CContext aContext, cMapID mID )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_DM, "%s %08X: '%s' '%s' %04X %d",
-                                  source->getName(), mID, mID->localID, mID->remoteID, mID->flags, mID->ident );
-  return DB_Forbidden; /* not yet implemented */
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    TSyError res = DB_Forbidden;
+    try {
+        if (source->getOperations().m_deleteMapItem) {
+            res = source->getOperations().m_deleteMapItem(mID);
+        }
+    } catch (...) {
+        res = source->handleException();
+    }
+
+    SE_LOG_DEBUG(source, NULL, "DeleteMapItem '%s' + %x = '%s' + %x res=%d",
+                 mID->localID, mID->ident,
+                 mID->remoteID, mID->flags,
+                 res);
+    return res;
 } /* DeleteMapItem */
 
 
@@ -519,9 +589,6 @@ TSyError SyncEvolution_DeleteMapItem( CContext aContext, cMapID mID )
 extern "C"
 void SyncEvolution_DisposeObj( CContext aContext, void* memory )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_Exotic_DB( source->getSynthesisAPI(), MyDB,Da_DO, "%s free at %08X", source->getName(),memory );
   free( memory );
 } /* DisposeObj */
 
@@ -530,10 +597,6 @@ void SyncEvolution_DisposeObj( CContext aContext, void* memory )
 extern "C"
 void SyncEvolution_ThreadMayChangeNow( CContext aContext )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  /* can be implemented empty, if no action is required */
-  SyncSource *source = DBC( aContext );
-  DEBUG_Exotic_DB( source->getSynthesisAPI(), MyDB,Da_TC, "%s", source->getName() );
 } /* ThreadMayChangeNow */
 
 
@@ -541,10 +604,6 @@ void SyncEvolution_ThreadMayChangeNow( CContext aContext )
 extern "C"
 void SyncEvolution_WriteLogData( CContext aContext, cAppCharP logData )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB, Da_WL, "%s (BEGIN)\n%s", source->getName(), logData );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB, Da_WL, "%s (END)",       source->getName() );
 } /* WriteLogData */
 
 
@@ -555,9 +614,6 @@ void SyncEvolution_WriteLogData( CContext aContext, cAppCharP logData )
 extern "C"
 void SyncEvolution_DispItems( CContext aContext, bool allFields, cAppCharP specificItem )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_DI, "%s %d '%s'", source->getName(),allFields,specificItem );
 } /* DispItems */
 
 
@@ -571,8 +627,11 @@ TSyError SyncEvolution_AdaptItem( CContext aContext, appCharP *aItemData1,
 { 
   /**** CAN BE ADAPTED BY USER ****/ 
   SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,"AdaptItem", "'%s' '%s' '%s' id=%d", 
-                          *aItemData1,*aItemData2,*aLocalVars, aIdentifier );
+  if (!source) {
+      return LOCERR_WRONGUSAGE;
+  }
+  SE_LOG_DEBUG(source, NULL, "AdaptItem '%s' '%s' '%s' id=%d", 
+               *aItemData1, *aItemData2, *aLocalVars, aIdentifier);
   return LOCERR_OK;
 } /* AdaptItem */
 
@@ -587,11 +646,15 @@ TSyError SyncEvolution_StartDataRead( CContext aContext, cAppCharP   lastToken,
                                       cAppCharP resumeToken )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_SR, "%s last='%s' resume='%s'",
-              source->getName(), lastToken,resumeToken );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     try {
+        BOOST_FOREACH(const SyncSource::Operations::CallbackFunctor_t &callback,
+                      source->getOperations().m_startAccess) {
+            callback();
+        }
         if (source->getOperations().m_startDataRead) {
             res = source->getOperations().m_startDataRead(lastToken, resumeToken);
         }
@@ -603,6 +666,8 @@ TSyError SyncEvolution_StartDataRead( CContext aContext, cAppCharP   lastToken,
         res = source->handleException();
     }
 
+    SE_LOG_DEBUG(source, NULL, "StartDataRead last='%s' resume='%s' res=%d",
+                 lastToken, resumeToken, res);
     return res;
 }
 
@@ -613,7 +678,9 @@ TSyError SyncEvolution_ReadNextItemAsKey( CContext aContext, ItemID aID, KeyH aI
 {
     /**** CAN BE ADAPTED BY USER ****/
     SyncSource *source = DBC( aContext );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     *aStatus = 0;
     memset(aID, 0, sizeof(*aID));
@@ -625,8 +692,8 @@ TSyError SyncEvolution_ReadNextItemAsKey( CContext aContext, ItemID aID, KeyH aI
         }
     }
 
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_RNK, "%s aStatus=%d aItemKey=%08X aID=(%s,%s)",
-              source->getName(), *aStatus, aItemKey, aID->item, aID->parent );
+    SE_LOG_DEBUG(source, NULL, "ReadNextItemAsKey aStatus=%d aID=(%s,%s) res=%d",
+                 *aStatus, aID->item, aID->parent, res);
     return res;
 }
 
@@ -634,9 +701,9 @@ extern "C"
 TSyError SyncEvolution_ReadItemAsKey( CContext aContext, cItemID aID, KeyH aItemKey )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_RIK, "%s aItemKey=%08X aID=(%s,%s)",
-              source->getName(), aItemKey,  aID->item,aID->parent );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     if (source->getOperations().m_readItemAsKey) {
         try {
@@ -646,43 +713,59 @@ TSyError SyncEvolution_ReadItemAsKey( CContext aContext, cItemID aID, KeyH aItem
         }
     }
 
+    SE_LOG_DEBUG(source, NULL, "ReadItemAsKey aID=(%s,%s) res=%d",
+                 aID->item, aID->parent, res);
     return res;
 }
 
-#if 0
 extern "C"
-TSyError SyncEvolution_ReadBlob( CContext aContext, cItemID  aID,  cAppCharP  aBlobID,
-                                 appPointer *aBlkPtr, uInt32 *aBlkSize, 
-                                 uInt32 *aTotSize,
-                                 bool  aFirst,    bool *aLast )
+sysync::TSyError SyncEvolution_ReadBlob(CContext aContext, cItemID  aID,  cAppCharP  aBlobID,
+                                        appPointer *aBlkPtr, memSize *aBlkSize, 
+                                        memSize *aTotSize,
+                                        bool  aFirst,    bool *aLast)
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
   SyncSource *source = DBC( aContext );
-  const int sz= sizeof(int);
-  
-  int* ip = (int*)malloc( sz ); /* example BLOB structure for test (=4 bytes) */ 
-      *ip = 231;
+  if (!source) {
+      return LOCERR_WRONGUSAGE;
+  }
 
-  *aBlkPtr = (appPointer)ip; if (*aBlkSize==0 || *aBlkSize>=sz) *aBlkSize= sz;
-  *aTotSize= *aBlkSize;
-  *aLast   = true;
+  TSyError res;
+  if (source->getOperations().m_readBlob) {
+      try {
+	    size_t blksize, totsize;
+	    /* Another conversion between memSize and size_t to make s390 happy */
+            res = source->getOperations().m_readBlob(aID, aBlobID, (void **)aBlkPtr,
+						     aBlkSize ? &blksize : NULL,
+						     aTotSize ? &totsize : NULL,
+						     aFirst, aLast);
+	    if (aBlkSize) {
+	        *aBlkSize = blksize;
+	    }
+            if (aTotSize) {
+                *aTotSize = totsize;
+            }
 
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_RB, "%s aID=(%s,%s) aBlobID=(%s)",
-                                  source->getName(), aID->item,aID->parent, aBlobID );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,"",    "aBlkPtr=%08X aBlkSize=%d aTotSize=%d aFirst=%s aLast=%s", 
-                                 *aBlkPtr, *aBlkSize, *aTotSize,
-                                  aFirst?"true":"false", *aLast?"true":"false" );
-  return LOCERR_OK;
+      } catch (...) {
+          res = source->handleException();
+      }
+  } else {
+      res = LOCERR_NOTIMP;
+  }
+
+  SE_LOG_DEBUG(source, NULL, "ReadBlob aID=(%s,%s) aBlobID=(%s) aBlkPtr=%p aBlkSize=%lu aTotSize=%lu aFirst=%s aLast=%s res=%d",
+               aID->item,aID->parent, aBlobID, aBlkPtr, (unsigned long)*aBlkSize, (unsigned long)*aTotSize,
+               aFirst? "true" : "false", *aLast ? "true" : "false", res);
+  return res;
 } /* ReadBlob */
-#endif
 
 
 extern "C"
 TSyError SyncEvolution_EndDataRead( CContext aContext )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_ER, "%s", source->getName() );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     if (source->getOperations().m_endDataRead) {
         try {
@@ -692,6 +775,7 @@ TSyError SyncEvolution_EndDataRead( CContext aContext )
         }
     }
 
+    SE_LOG_DEBUG(source, NULL, "EndDataRead res=%d", res);
     return res;
 }
 
@@ -703,7 +787,10 @@ extern "C"
 TSyError SyncEvolution_StartDataWrite( CContext aContext )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_SW, "%s", source->getName() );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(source, NULL, "StartDataWrite");
     return LOCERR_OK;
 }
 
@@ -712,8 +799,9 @@ TSyError SyncEvolution_InsertItemAsKey( CContext aContext, KeyH aItemKey, ItemID
 {
     /**** CAN BE ADAPTED BY USER ****/
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_IIK, "%s %08X\n", source->getName(), aItemKey );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     if (source->getOperations().m_insertItemAsKey) {
         try {
@@ -723,6 +811,7 @@ TSyError SyncEvolution_InsertItemAsKey( CContext aContext, KeyH aItemKey, ItemID
         }
     }
 
+    SE_LOG_DEBUG(source, NULL, "InsertItemAsKey res=%d\n", res);
     return res;
 }
 
@@ -732,10 +821,9 @@ TSyError SyncEvolution_UpdateItemAsKey( CContext aContext, KeyH aItemKey, cItemI
                                         ItemID updID )
 {
     SyncSource *source = DBC( aContext );
-  
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_UI, "%s aID=(%s,%s)",
-              source->getName(), aID->item,aID->parent );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     if (source->getOperations().m_updateItemAsKey) {
         try {
@@ -745,6 +833,9 @@ TSyError SyncEvolution_UpdateItemAsKey( CContext aContext, KeyH aItemKey, cItemI
         }
     }
 
+  
+    SE_LOG_DEBUG(source, NULL, "aID=(%s,%s) res=%d",
+                 aID->item,aID->parent, res);
     return res;
 }
 
@@ -754,9 +845,11 @@ extern "C"
 TSyError SyncEvolution_MoveItem( CContext aContext, cItemID aID, cAppCharP newParID )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_MvI, "%s aID=(%s,%s) => (%s,%s)",
-              source->getName(), aID->item,aID->parent,
-              aID->item,newParID );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(source, NULL, "MoveItem aID=(%s,%s) => (%s,%s)",
+                 aID->item,aID->parent, aID->item,newParID);
     return LOCERR_NOTIMP;
 }
 
@@ -766,8 +859,9 @@ extern "C"
 TSyError SyncEvolution_DeleteItem( CContext aContext, cItemID aID )
 {
     SyncSource *source = DBC( aContext );
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_DeI, "%s aID=(%s,%s)", source->getName(),aID->item,aID->parent );
-
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     if (source->getOperations().m_deleteItem) {
         try {
@@ -777,6 +871,8 @@ TSyError SyncEvolution_DeleteItem( CContext aContext, cItemID aID )
         }
     }
 
+    SE_LOG_DEBUG(source, NULL, "DeleteItem aID=(%s,%s) res=%d",
+                 aID->item, aID->parent, res);
     return res;
 }
 
@@ -785,6 +881,11 @@ TSyError SyncEvolution_DeleteItem( CContext aContext, cItemID aID )
 extern "C"
 TSyError SyncEvolution_FinalizeLocalID( CContext aContext, cItemID aID, ItemID updID )
 {
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(source, NULL, "FinalizeLocalID not implemented");
     return LOCERR_NOTIMP;
 }
 
@@ -793,45 +894,76 @@ TSyError SyncEvolution_FinalizeLocalID( CContext aContext, cItemID aID, ItemID u
 extern "C"
 TSyError SyncEvolution_DeleteSyncSet( CContext aContext )
 {
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+    SE_LOG_DEBUG(source, NULL, "DeleteSyncSet not implemented");
     return LOCERR_NOTIMP;
 }
 
 
-#if 0
 extern "C"
-TSyError SyncEvolution_WriteBlob( CContext aContext, cItemID aID,  cAppCharP aBlobID,
-                                  appPointer aBlkPtr, uInt32 aBlkSize, 
-                                  uInt32 aTotSize,
-                                  bool aFirst,    bool aLast )
+TSyError SyncEvolution_WriteBlob(CContext aContext, cItemID aID,  cAppCharP aBlobID,
+                                 appPointer aBlkPtr, memSize aBlkSize, 
+                                 memSize aTotSize,
+                                 bool aFirst,    bool aLast)
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_WB, "%s aID=(%s,%s) aBlobID=(%s)",
-                                  source->getName(), aID->item,aID->parent, aBlobID );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,"",    "aBlkPtr=%08X aBlkSize=%d aTotSize=%d aFirst=%s aLast=%s", 
-                                  aBlkPtr, aBlkSize, aTotSize, 
-                                  aFirst?"true":"false", aLast ?"true":"false" );
-  return LOCERR_OK;
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+
+    TSyError res;
+    if (source->getOperations().m_writeBlob) {
+        try {
+            res = source->getOperations().m_writeBlob(aID, aBlobID, aBlkPtr, aBlkSize,
+                                                      aTotSize, aFirst, aLast);
+        } catch (...) {
+            res = source->handleException();
+        }
+    } else {
+        res = LOCERR_NOTIMP;
+    }
+    
+    SE_LOG_DEBUG(source, NULL, "WriteBlob aID=(%s,%s) aBlobID=(%s) aBlkPtr=%p aBlkSize=%lu aTotSize=%lu aFirst=%s aLast=%s res=%d",
+                 aID->item,aID->parent, aBlobID, aBlkPtr, (unsigned long)aBlkSize, (unsigned long)aTotSize, 
+                 aFirst ? "true" : "false", aLast ? "true" : "false", res);
+    return res;
 } /* WriteBlob */
 
 
 extern "C"
 TSyError SyncEvolution_DeleteBlob( CContext aContext, cItemID aID, cAppCharP aBlobID )
 {
-  /**** CAN BE ADAPTED BY USER ****/ 
-  SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_DB, "%s aID=(%s,%s) aBlobID=(%s)",
-                                  source->getName(), aID->item,aID->parent, aBlobID );
-  return LOCERR_OK;
+    SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
+
+    TSyError res;
+    if (source->getOperations().m_deleteBlob) {
+        try {
+            res = source->getOperations().m_deleteBlob(aID, aBlobID);
+        } catch (...) {
+            res = source->handleException();
+        }
+    } else {
+        res = LOCERR_NOTIMP;
+    }
+
+    SE_LOG_DEBUG(source, NULL, "DeleteBlob aID=(%s,%s) aBlobID=(%s) res=%d",
+                 aID->item,aID->parent, aBlobID, res);
+    return res;
 } /* DeleteBlob */
 
-#endif
-
-
 extern "C"
 TSyError SyncEvolution_EndDataWrite( CContext aContext, bool success, appCharP *newToken )
 {
     SyncSource *source = DBC( aContext );
+    if (!source) {
+        return LOCERR_WRONGUSAGE;
+    }
     TSyError res = LOCERR_OK;
     try {
         BOOST_FOREACH(const SyncSource::Operations::CallbackFunctor_t &callback,
@@ -845,20 +977,21 @@ TSyError SyncEvolution_EndDataWrite( CContext aContext, bool success, appCharP *
         res = source->handleException();
     }
 
-    DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_EW, "%s %s '%s'", 
-              source->getName(), success ? "COMMIT":"ROLLBACK", *newToken );
+    SE_LOG_DEBUG(source, NULL, "EndDataWrite %s '%s' res=%d", 
+                 success ? "COMMIT":"ROLLBACK", *newToken, res);
     return res;
 }
 
-
-
 /* ----------------------------------- */
 extern "C"
 TSyError SyncEvolution_DeleteContext( CContext aContext )
 {
   /**** CAN BE ADAPTED BY USER ****/ 
   SyncSource *source = DBC( aContext );
-  DEBUG_DB( source->getSynthesisAPI(), MyDB,Da_DC, "%s", source->getName() );
+  if (!source) {
+      return LOCERR_WRONGUSAGE;
+  }
+  SE_LOG_DEBUG(source, NULL, "DeleteContext");
   source->popSynthesisAPI();
   return LOCERR_OK;
 }
