@@ -120,6 +120,15 @@ class TrackingSyncSource : public TestingSyncSource,
      */
     virtual bool isEmpty() = 0;
 
+    /**
+     * A unique identifier for the current state of the complete database.
+     * The semantic is the following:
+     * - empty string implies "state unknown" or "identifier not supported" (the default implementation)
+     * - id not empty and id_1 == id_2 implies "nothing has changed";
+     *   the inverse is not true (ids may be different although nothing has changed)
+     */
+    virtual std::string databaseRevision() { return ""; }
+
     /**
      * fills the complete mapping from LUID to revision string of all
      * currently existing items
@@ -134,6 +143,32 @@ class TrackingSyncSource : public TestingSyncSource,
      */
     virtual void listAllItems(SyncSourceRevisions::RevisionMap_t &revisions) = 0;
 
+    /**
+     * Called at the start of the sync session to tell
+     * the derived class about the cached information if (and only
+     * if) listAllItems() and updateAllItems() were not called. The derived class
+     * might not need this information, so the default implementation
+     * simply ignores.
+     *
+     * A more complex API could have been defined to only prepare the
+     * information when needed, but that seemed unnecessarily complex.
+     */
+    virtual void setAllItems(const RevisionMap_t &revisions) {}
+
+    /**
+     * updates the revision map to reflect the current state
+     *
+     * May be called instead of listAllItems() if the caller has
+     * a valid list to start from. If the implementor
+     * cannot update the list, it must start from scratch by
+     * reseting the list and calling listAllItems(). The default
+     * implementation of this method does that.
+     */
+    virtual void updateAllItems(SyncSourceRevisions::RevisionMap_t &revisions) {
+        revisions.clear();
+        listAllItems(revisions);
+    }
+
     /**
      * Create or modify an item.
      *
@@ -199,19 +234,31 @@ class TrackingSyncSource : public TestingSyncSource,
      * Returns the preferred mime type of the items handled by the sync source.
      * Example: "text/x-vcard"
      */
-    virtual const char *getMimeType() const = 0;
+    virtual std::string getMimeType() const = 0;
 
     /**
      * Returns the version of the mime type used by client.
      * Example: "2.1"
      */
-    virtual const char *getMimeVersion() const = 0;
+    virtual std::string getMimeVersion() const = 0;
 
     using SyncSource::getName;
 
   private:
     void checkStatus(SyncSourceReport &changes);
+    boost::shared_ptr<ConfigNode> m_trackingNode;
 
+    /**
+     * Stores meta information besides the item list:
+     * - "databaseRevision" = result of databaseRevision() at end of last sync
+     *
+     * Shares the same key/value store as m_trackingNode,
+     * which uses the "item-" prefix in its keys to
+     * avoid name clashes.
+     */
+    boost::shared_ptr<ConfigNode> m_metaNode;
+
+ protected:
     /* implementations of SyncSource callbacks */
     virtual void beginSync(const std::string &lastToken, const std::string &resumeToken);
     virtual std::string endSync(bool success);
@@ -222,9 +269,7 @@ class TrackingSyncSource : public TestingSyncSource,
     virtual void readItemRaw(const std::string &luid, std::string &item);
     virtual void enableServerMode();
     virtual bool serverModeEnabled() const;
-    virtual const char *getPeerMimeType() const;
-
-    boost::shared_ptr<ConfigNode> m_trackingNode;
+    virtual std::string getPeerMimeType() const;
 };
 
 
