@@ -28,6 +28,7 @@
 #include <boost/function.hpp>
 
 #include <stdarg.h>
+#include <time.h>
 
 #include <vector>
 #include <sstream>
@@ -36,6 +37,8 @@
 #include <exception>
 #include <list>
 
+#include <syncevo/Logging.h>
+
 #include <syncevo/declarations.h>
 SE_BEGIN_CXX
 using namespace std;
@@ -181,6 +184,7 @@ class StringEscape
 {
  public:
     enum Mode {
+        SET,               /**< explicit list of characters to be escaped */
         INI_VALUE,         /**< right hand side of .ini assignment:
                               escape all spaces at start and end (but not in the middle) and the equal sign */
         INI_WORD,          /**< same as before, but keep it one word:
@@ -192,6 +196,7 @@ class StringEscape
  private:
     char m_escapeChar;
     Mode m_mode;
+    std::set<char> m_forbidden;
 
  public:
     /**
@@ -203,6 +208,12 @@ class StringEscape
         m_mode(mode)
     {}
 
+    /**
+     * @param escapeChar        character used to introduce escape sequence
+     * @param forbidden         explicit list of characters which are to be escaped
+     */
+    StringEscape(char escapeChar, const char *forbidden);
+
     /** special character which introduces two-char hex encoded original character */
     char getEscapeChar() const { return m_escapeChar; }
     void setEscapeChar(char escapeChar) { m_escapeChar = escapeChar; }
@@ -213,7 +224,7 @@ class StringEscape
     /**
      * escape string according to current settings
      */
-    string escape(const string &str) const { return escape(str, m_escapeChar, m_mode); }
+    string escape(const string &str) const;
 
     /** escape string with the given settings */
     static string escape(const string &str, char escapeChar, Mode mode);
@@ -245,6 +256,15 @@ class UUID : public string {
     UUID();
 };
 
+/**
+ * Safety check for string pointer.
+ * Returns pointer if valid, otherwise the default string.
+ */
+inline const char *NullPtrCheck(const char *ptr, const char *def = "(null)")
+{
+    return ptr ? ptr : def;
+}
+
 /**
  * A C++ wrapper around readir() which provides the names of all
  * directory entries, excluding . and ..
@@ -306,6 +326,63 @@ std::string StringPrintf(const char *format, ...)
 ;
 std::string StringPrintfV(const char *format, va_list ap);
 
+/**
+ * strncpy() which inserts adds 0 byte
+ */
+char *Strncpy(char *dest, const char *src, size_t n);
+
+/**
+ * sleep() with sub-second resolution. Might be interrupted by signals
+ * before the time has elapsed.
+ */
+void Sleep(double seconds);
+
+
+/**
+ * Sub-second time stamps. Thin wrapper around timespec
+ * and clock_gettime() (for monotonic time). Comparisons
+ * assume normalized values (tv_nsec >= 0, < 1e9). Addition
+ * and substraction produce normalized values, as long
+ * as the result is positive. Substracting a - b where a < b
+ * leads to an undefined result.
+ */
+class Timespec : public timespec
+{
+ public:
+    Timespec() { tv_sec = 0; tv_nsec = 0; }
+    Timespec(time_t sec, long nsec) { tv_sec = sec; tv_nsec = nsec; }
+
+    bool operator < (const Timespec &other) const {
+        return tv_sec < other.tv_sec ||
+            (tv_sec == other.tv_sec && tv_nsec < other.tv_nsec);
+    }
+    bool operator > (const Timespec &other) const {
+        return tv_sec > other.tv_sec ||
+            (tv_sec == other.tv_sec && tv_nsec > other.tv_nsec);
+    }
+    bool operator <= (const Timespec &other) const { return !(*this > other); }
+    bool operator >= (const Timespec &other) const { return !(*this < other); }
+
+    operator bool () const { return tv_sec || tv_nsec; }
+
+    Timespec operator + (int seconds) const { return Timespec(tv_sec + seconds, tv_nsec); }
+    Timespec operator - (int seconds) const { return Timespec(tv_sec - seconds, tv_nsec); }
+    Timespec operator + (unsigned seconds) const { return Timespec(tv_sec + seconds, tv_nsec); }
+    Timespec operator - (unsigned seconds) const { return Timespec(tv_sec - seconds, tv_nsec); }
+    Timespec operator + (const Timespec &other) const;
+    Timespec operator - (const Timespec &other) const;
+
+    operator timeval () const { timeval res; res.tv_sec = tv_sec; res.tv_usec = tv_nsec / 1000; return res; }
+
+    time_t seconds() const { return tv_sec; }
+    long nsecs() const { return tv_nsec; }
+    double duration() const { return (double)tv_sec + ((double)tv_nsec) / 1e9;  }
+
+    static Timespec monotonic() { Timespec res; clock_gettime(CLOCK_MONOTONIC, &res); return res; }
+    static Timespec system() { Timespec res; clock_gettime(CLOCK_REALTIME, &res); return res; }
+};
+ 
+
 /**
  * an exception which records the source file and line
  * where it was thrown
@@ -329,15 +406,21 @@ class Exception : public std::runtime_error
     /**
      * Convenience function, to be called inside a catch(..) block.
      *
-     * Rethrows the exception to determine what it is, then logs it as
-     * an error. Turns certain known exceptions into the corresponding
+     * Rethrows the exception to determine what it is, then logs it
+     * at the chosen level (error by default).
+     *
+     * Turns certain known exceptions into the corresponding
      * status code if status still was STATUS_OK when called.
      * Returns updated status code.
      *
      * @param logger    the class which does the logging
+     * @retval explanation   set to explanation for problem, if non-NULL
+     * @param level     level to be used for logging
      */
-    static SyncMLStatus handle(SyncMLStatus *status = NULL, Logger *logger = NULL);
+    static SyncMLStatus handle(SyncMLStatus *status = NULL, Logger *logger = NULL, std::string *explanation = NULL, Logger::Level = Logger::ERROR);
     static SyncMLStatus handle(Logger *logger) { return handle(NULL, logger); }
+    static SyncMLStatus handle(std::string &explanation) { return handle(NULL, NULL, &explanation); }
+    static void log() { handle(NULL, NULL, NULL, Logger::DEBUG); }
 };
 
 /**
@@ -358,6 +441,27 @@ protected:
     SyncMLStatus m_status;
 };
 
+class TransportException : public Exception
+{
+ public:
+    TransportException(const std::string &file,
+                       int line,
+                       const std::string &what) :
+    Exception(file, line, what) {}
+    ~TransportException() throw() {}
+};
+
+class TransportStatusException : public StatusException
+{
+ public:
+    TransportStatusException(const std::string &file,
+                             int line,
+                             const std::string &what,
+                             SyncMLStatus status) :
+    StatusException(file, line, what, status) {}
+    ~TransportStatusException() throw() {}
+};
+
 /**
  * replace ${} with environment variables, with
  * XDG_DATA_HOME, XDG_CACHE_HOME and XDG_CONFIG_HOME having their normal
@@ -376,6 +480,23 @@ inline string getHome() {
  * */
 std::vector<std::string> unescapeJoinedString (const std::string &src, char separator);
 
+/**
+ * mapping from int flag to explanation
+ */
+struct Flag {
+    int m_flag;
+    const char *m_description;
+};
+
+/**
+ * turn flags into comma separated list of explanations
+ *
+ * @param flags     bit mask
+ * @param descr     array with zero m_flag as end marker
+ * @param sep       used to join m_description strings
+ */
+std::string Flags2String(int flags, const Flag *descr, const std::string &sep = ", ");
+
 /**
  * Temporarily set env variable, restore old value on destruction.
  * Useful for unit tests which depend on the environment.
@@ -400,6 +521,22 @@ std::string getCurrentTime();
 #define SE_THROW_EXCEPTION(_class,  _what) \
     throw _class(__FILE__, __LINE__, _what)
 
+/** throw a class which accepts file, line, what plus 1 additional parameter */
+#define SE_THROW_EXCEPTION_1(_class,  _what, _x1)   \
+    throw _class(__FILE__, __LINE__, (_what), (_x1))
+
+/** throw a class which accepts file, line, what plus 2 additional parameters */
+#define SE_THROW_EXCEPTION_2(_class,  _what, _x1, _x2) \
+    throw _class(__FILE__, __LINE__, (_what), (_x1), (_x2))
+
+/** throw a class which accepts file, line, what plus 2 additional parameters */
+#define SE_THROW_EXCEPTION_3(_class,  _what, _x1, _x2, _x3) \
+    throw _class(__FILE__, __LINE__, (_what), (_x1), (_x2), (_x3))
+
+/** throw a class which accepts file, line, what plus 2 additional parameters */
+#define SE_THROW_EXCEPTION_4(_class,  _what, _x1, _x2, _x3, _x4) \
+    throw _class(__FILE__, __LINE__, (_what), (_x1), (_x2), (_x3), (_x4))
+
 /** throw a class which accepts file, line, what parameters and status parameters*/
 #define SE_THROW_EXCEPTION_STATUS(_class,  _what, _status) \
     throw _class(__FILE__, __LINE__, _what, _status)
