@@ -19,18 +19,25 @@
  02110-1301  USA
 '''
 import sys,os,glob,datetime,popen2
+import re
 
 """ 
 resultcheck.py: tranverse the test result directory, generate an XML
 based test report.
 """
 
+# sort more accurately on sub-second modification times
+os.stat_float_times(True)
+
 space="  "
 def check (resultdir, serverlist,resulturi, srcdir, shellprefix, backenddir):
     '''Entrypoint, resutldir is the test result directory to be generated,
     resulturi is the http uri, it will only process corresponding server's
     test results list in severlist'''
-    servers = serverlist.split(",")
+    if serverlist:
+        servers = serverlist.split(",")
+    else:
+        servers = []
     result = open("nightly.xml","w")
     result.write('''<?xml version="1.0" encoding="utf-8" ?>\n''')
     result.write('''<nightly-test>\n''')
@@ -41,6 +48,10 @@ def check (resultdir, serverlist,resulturi, srcdir, shellprefix, backenddir):
         indents,cont = step1(resultdir+"/output.txt",result,indents,resultdir,resulturi, shellprefix, srcdir)
         if (cont):
             step2(resultdir,result,servers,indents,srcdir,shellprefix,backenddir)
+        else:
+            # compare.xsl fails if there is no <client-test> element:
+            # add an empty one
+            result.write('''<client-test/>\n''')
     result.write('''</nightly-test>\n''')
     result.close()
 
@@ -48,10 +59,6 @@ def step1(input, result, indents, dir, resulturi, shellprefix, srcdir):
     '''Step1 of the result checking, collect system information and 
     check the preparation steps (fetch, compile)'''
     cont = True
-    # get the chroot envoriment
-    root = ''
-    if(shellprefix.split()[0] == 'schroot'):
-        root = shellprefix.split()[3]
     indent =indents[-1]+space
     indents.append(indent)
     result.write(indent+'''<platform-info>\n''')
@@ -72,11 +79,13 @@ def step1(input, result, indents, dir, resulturi, shellprefix, srcdir):
     s = fout.read()
     result.write(indent+s)
     result.write(indent+'''</osinfo>\n''')
-    if(len(root) > 0):
+    if 'schroot' in shellprefix:
         result.write(indent+'''<chrootinfo>\n''')
-        fout,fin=popen2.popen2('schroot -i -c'+root+" |grep 'Description'")
-        s = fout.read()
-        s = s + ' name: '+root;
+        fout,fin=popen2.popen2(shellprefix.replace('schroot', 'schroot -i'))
+        s = ""
+        for line in fout:
+            if line.startswith("  Name ") or line.startswith("  Description "):
+                 s = s + line
         result.write(indent+s)
         result.write(indent+'''</chrootinfo>\n''')
     result.write(indent+'''<libraryinfo>\n''')
@@ -137,18 +146,20 @@ def step2(resultdir, result, servers, indents, srcdir, shellprefix, backenddir):
     servers, tranverse the corresponding result folder, process
     each log file to decide the status of the testcase'''
     '''Read the runtime parameter for each server '''
-    cmd='sed -n '
-    for server in servers:
-        cmd+= '-e /^'+server+'/p '
-    fout,fin=popen2.popen2(cmd +resultdir+'/output.txt')
-    params={}
-    for line in fout:
+    params = {}
+    if servers:
+        cmd='sed -n '
         for server in servers:
-            if(line.startswith(server) and server not in params):
-                t = line.partition(server)[2].rpartition('\n')[0]
-                if(t.startswith(':')):
-                    t=t.partition(':')[2]
-                params[server]=t
+            cmd+= '-e /^'+server+'/p '
+        fout,fin=popen2.popen2(cmd +resultdir+'/output.txt')
+        for line in fout:
+            for server in servers:
+                # find first line with "foobar successful" or "foobar: <command failure>"
+                if (line.startswith(server + ":") or line.startswith(server + " ")) and server not in params:
+                    t = line.partition(server)[2].rpartition('\n')[0]
+                    if(t.startswith(':')):
+                        t=t.partition(':')[2]
+                    params[server]=t
     
     indent =indents[-1]+space
     indents.append(indent)
@@ -157,7 +168,12 @@ def step2(resultdir, result, servers, indents, srcdir, shellprefix, backenddir):
     runservers = os.listdir(resultdir)
     #list source test servers statically, we have no idea how to differenciate
     #automatically whether the server is a source test or sync test.
-    sourceServers = ['evolution', 'evolution-prebuilt-build']
+    sourceServers = ['evolution',
+                     'evolution-prebuilt-build',
+                     'yahoo',
+                     'googlecalendar',
+                     'apple',
+                     'dbus']
     sourceServersRun = 0
     haveSource = False
     #Only process servers listed in the input parameter and in the sourceServer
@@ -201,10 +217,10 @@ def step2(resultdir, result, servers, indents, srcdir, shellprefix, backenddir):
                 templates=[]
                 oldpath = os.getcwd()
                 os.chdir (srcdir)
-                fout,fin=popen2.popen2(shellprefix + " env LD_LIBRARY_PATH=build-synthesis/src/.libs SYNCEVOLUTION_BACKEND_DIR="+backenddir +" ./client-test -h |grep 'Client::Sync::vcard21'|grep -v 'Retry' |grep -v 'Suspend' | grep -v 'Resend'")
+                fout,fin=popen2.popen2(shellprefix + " env LD_LIBRARY_PATH=build-synthesis/src/.libs SYNCEVOLUTION_BACKEND_DIR="+backenddir +" CLIENT_TEST_SOURCES=file_contact ./client-test -h |grep 'Client::Sync::file_contact'|grep -v 'Retry' |grep -v 'Suspend' | grep -v 'Resend'")
                 os.chdir(oldpath)
                 for line in fout:
-                    l = line.partition('Client::Sync::vcard21::')[2].rpartition('\n')[0]
+                    l = line.partition('Client::Sync::file_contact::')[2].rpartition('\n')[0]
                     if(l!=''):
                         templates.append(l);
                 indent +=space
@@ -221,30 +237,116 @@ def step2(resultdir, result, servers, indents, srcdir, shellprefix, backenddir):
             if(params[server].find('return code ') !=-1):
                 result.write('result="'+params[server].partition('return code ')[2].partition(')')[0]+'" ')
             result.write('>\n')
-            logs = glob.glob(resultdir+'/'+rserver+'/*.log')
+            # sort files by creation time, to preserve run order
+            logs = map(lambda file: (os.stat(file).st_mtime, file),
+                       glob.glob(resultdir+'/'+rserver+'/*.log'))
+            logs.sort()
+            logs = map(lambda entry: entry[1], logs)
             logdic ={}
             logprefix ={}
-            for log in logs:
-                if(log.endswith('____compare.log')):
-                    continue
-                if(len(log.split('_')) > 3):
-                    format = log.rpartition('_')[0].partition('_')[2].partition('_')[2]
-                    prefix = log.rpartition(format)[0].rpartition('/')[-1]
-                elif (len(log.split('_')) == 3):
-                    format = log.rpartition('_')[0].partition('_')[2]
-                    prefix = log.rpartition(format)[0].rpartition('/')[-1]
-                else:
-                    format = log.partition('_')[0].rpartition('/')[-1]
-                    prefix = ''
-                if(format not in logdic):
-                    logdic[format]=[]
-                logdic[format].append(log)
-                logprefix[format]=prefix
+            if server == 'dbus':
+                # Extract tests and their results from output.txt,
+                # which contains Python unit test output. Example follows.
+                # Note that there can be arbitrary text between the test name
+                # and "ok" resp. "FAIL/ERROR". Therefore failed tests
+                # are identified not by those words but rather by the separate
+                # error reports at the end of the output. Those reports
+                # are split out into separate .log files for easy viewing
+                # via the .html report.
+                #
+                # TestDBusServer.testCapabilities - Server.Capabilities() ... ok
+                # TestDBusServer.testGetConfigScheduleWorld - Server.GetConfigScheduleWorld() ... ok
+                # TestDBusServer.testGetConfigsEmpty - Server.GetConfigsEmpty() ... ok
+                # TestDBusServer.testGetConfigsTemplates - Server.GetConfigsTemplates() ... FAIL
+                # TestDBusServer.testInvalidConfig - Server.NoSuchConfig exception ... ok
+                # TestDBusServer.testVersions - Server.GetVersions() ... ok
+                #
+                #======================================================================
+                # FAIL: TestDBusServer.testGetConfigsTemplates - Server.GetConfigsTemplates()
+                # ---------------------------------------------------------------------
+
+                # first build list of all tests, assuming that they pass
+                dbustests = {}
+                test_start = re.compile(r'''^Test(?P<cl>.*)\.test(?P<func>[^ ]*)''')
+                test_fail = re.compile(r'''(?P<type>FAIL|ERROR): Test(?P<cl>.*)\.test(?P<func>[^ ]*)''')
+                logfile = None
+                sepcount = 0
+                for line in open(rserver + "/output.txt"):
+                    m = test_start.search(line)
+                    if m:
+                        is_okay = True
+                    else:
+                        m = test_fail.search(line)
+                        is_okay = False
+                    if m:
+                        # create log file
+                        cl = m.group("cl")
+                        func = m.group("func")
+                        name = rserver + "/" + cl + "_" + func + ".log"
+                        logfile = open(name, "w")
+                        if not dbustests.get(cl):
+                            dbustests[cl] = {}
+                        if is_okay:
+                            # okay: write a single line with the full test description
+                            dbustests[cl][func] = "okay"
+                            logfile.write(line)
+                            logfile = None
+                        else:
+                            # failed: start writing lines into separate log file
+                            dbustests[cl][func] = m.group("type")
+                            sepcount = 0
+                    if logfile:
+                        logfile.write(line)
+                        if line.startswith("-----------------------------------"):
+                            sepcount = sepcount + 1
+                            if sepcount >= 2:
+                                # end of failure output for this test
+                                logfile = None
+                # now write XML
+                indent +=space
+                indents.append(indent)
+                for testclass in dbustests:
+                    result.write('%s<%s prefix="">\n' %
+                                 (indent, testclass))
+                    indent +=space
+                    indents.append(indent)
+                    for testfunc in dbustests[testclass]:
+                        result.write('%s<%s>%s</%s>\n' %
+                                     (indent, testfunc,
+                                      dbustests[testclass][testfunc], 
+                                      testfunc))
+                    indents.pop()
+                    indent = indents[-1]
+                    result.write('%s</%s>\n' %
+                                 (indent, testclass))
+                indents.pop()
+                indent = indents[-1]
+            else:
+                for log in logs:
+                    if os.path.basename(log) in ['____compare.log',
+                                                 'syncevo.log', # D-Bus server output
+                                                 'dbus.log', # dbus-monitor output
+                                                 ]:
+                        continue
+                    # <path>/Client_Sync_eds_contact_testItems.log
+                    # <path>/SyncEvo_CmdlineTest_testConfigure.log
+                    # <path>/N7SyncEvo11CmdlineTestE_testConfigure.log - C++ name mangling?
+                    m = re.match(r'.*/(Client_Source_|Client_Sync_|N7SyncEvo\d+|[^_]*_)(.*)_([^_]*)', log)
+                    # Client_Sync_, Client_Source_, SyncEvo_, ...
+                    prefix = m.group(1)
+                    # eds_contact, CmdlineTest, ...
+                    format = m.group(2)
+                    if(format not in logdic):
+                        logdic[format]=[]
+                    logdic[format].append(log)
+                    logprefix[format]=prefix
             for format in logdic.keys():
                 indent +=space
                 indents.append(indent)
                 prefix = logprefix[format]
                 qformat = format;
+                # avoid + sign in element name (not allowed by XML);
+                # code reading XML must replace _- with + and __ with _
                 qformat = qformat.replace("_", "__");
                 qformat = qformat.replace("+", "_-");
                 result.write(indent+'<'+qformat+' prefix="'+prefix+'">\n')
